(* tag: Abelian monoids *)

(** * Abelian monoids *)

Require Export cmonoid_.

Implicit Arguments On.

Module AbMonoid_ <: Cat_Sig_.

Export Misc_.

Section Monoid_Theory.

Variable A:Type; add:A->A->A; zero:A.

Record Monoid_Theory : Prop := {
  ax_add_assoc : (x,y,z:A) (add x (add y z))==(add (add x y) z);
  ax_add_commut : (x,y:A) (add x y)==(add y x);
  ax_zero_sx : (x:A) (add zero x)==x;
  ax_zero_dx : (x:A) (add x zero)==x
 }.

End Monoid_Theory.

Record Monoid_Data : Type := {
  monoid_carrier :> Type;
  add : monoid_carrier->monoid_carrier->monoid_carrier;
  zero : monoid_carrier
 }.

Implicits zero [1].

Record Monoid : Type := {
  monoid_data :> Monoid_Data;
  monoid_theory :>
   (Monoid_Theory (!add monoid_data) (!zero monoid_data))
 }.

Definition Obj := Monoid.

Section Monoid_Facts.

Variable A:Monoid.

Lemma add_assoc : (x,y,z:A) (add x (add y z))==(add (add x y) z).
Proof.
  Intros; Apply (ax_add_assoc A).
Qed.

Lemma add_commut : (x,y:A) (add x y)==(add y x).
Proof.
  Intros; Apply (ax_add_commut A).
Qed.

Lemma zero_sx : (x:A) (add zero x)==x.
Proof.
  Intros; Apply (ax_zero_sx A).
Qed.

Lemma zero_dx : (x:A) (add x zero)==x.
Proof.
  Intros; Apply (ax_zero_dx A).
Qed.

End Monoid_Facts.

Hints Resolve add_assoc add_commut zero_sx zero_dx.

Record Morph_Theory [A,B:Monoid; f:A->B] : Prop := {
  morph_add : (x,y:A) (f (add x y))==(add (f x) (f y));
  morph_zero : (f zero)==zero
 }.

Record Monoid_Morph [A,B:Monoid] : Type := {
  morph_data :> A->B;
  morph_theory :> (Morph_Theory morph_data)
 }.

Definition Morph [A,B:Monoid] : Type := (Monoid_Morph A B).

Lemma morph_extens : (A,B:Monoid; f,g:(Morph A B))
 ((x:A)(f x)==(g x)) -> f==g.
Proof.
  NewDestruct f; NewDestruct g; Simpl; Intros.
  Assert U : morph_data0 == morph_data1.
  Apply nondep_extensionality; Assumption.
  NewDestruct U.
  Assert U : morph_theory0 == morph_theory1.
  Apply proof_irrelevance.
  NewDestruct U.
  Reflexivity.
Qed.

Section morph_comp.

Variable A,B,C:Monoid.

Variable f:(Morph B C); g:(Morph A B).

Local h [x:A] : C := (f (g x)).

Remark h_th : (Morph_Theory h).
Proof.
  Unfold h; Split; Intros.
  Rewrite (morph_add g).
  Rewrite (morph_add f).
  Reflexivity.
  Rewrite (morph_zero g).
  Rewrite (morph_zero f).
  Reflexivity.
Qed.

Definition morph_comp : (Morph A C).
Proof.
  Split with h;
  Apply h_th.
Defined.

End morph_comp.

Definition morph_id [A:Monoid] : (Morph A A).
Proof.
  Intros.
  Split with [x:A]x.
  Abstract Split; Auto.
Defined.

Implicits morph_id [1].

Lemma morph_comp_assoc : (A,B,C,D:Monoid)
 (h:(Morph C D); g:(Morph B C); f:(Morph A B))
 (morph_comp h (morph_comp g f))==(morph_comp (morph_comp h g) f).
Proof.
  Intros; Apply morph_extens; Trivial.
Qed.

Lemma morph_id_sx : (A,B:Monoid; f:(Morph A B))
 (morph_comp morph_id f)==f.
Proof.
  Intros; Apply morph_extens; Trivial.
Qed.

Lemma morph_id_dx : (A,B:Monoid; f:(Morph A B))
 (morph_comp f morph_id)==f.
Proof.
  Intros; Apply morph_extens; Trivial.
Qed.

Definition cmonoid_of_abmonoid [A:Monoid] : CMonoid_.Monoid.
Proof.
  Intros.
  Split with (CMonoid_.Build_Monoid_Data (!add A) (!zero A)).
  Abstract NewDestruct (!monoid_theory A); Split; Simpl; Auto.
Defined.

Definition abmonoid_of_cmonoid [A:CMonoid_.Monoid] : Monoid.
Proof.
  Intros.
  Split with (Build_Monoid_Data (!CMonoid_.mul A) (!CMonoid_.one A)).
  Abstract NewDestruct (!CMonoid_.monoid_theory A); Split; Simpl; Auto.
Defined.

End AbMonoid_.
