(* tag: Categories *)

Require misc.
Require map.
Require cat_sig.

Implicit Arguments On.

(** * Categories. *)

(** Development of Category Theory. *)

(** Notations and conventions:
  - [Cat] : the type of a category.
  - [(mor x y)] : type of the morphisms between two objects [x], [y].
  - [(oo g f)]	: composition of two morphisms.
  - [(ii x)] : identity morphism of the object [x].
  - [(Functor A B)] : type of the functor between two categoriess [A], [B].
  - [(NT F G)] : type of natural transformations between two functors [F], [G].
*)

Module Cat.

Import Map.
Import Misc.

(** ** Definition of category *)

Section Cat_Theory.

Variable A:Type; mor:A->A->Type.
Variable oo:(x,y,z:A)(mor y z)->(mor x y)->(mor x z).
Variable ii:(x:A)(mor x x).

Record Cat_Theory : Prop := {
  ax_oo_assoc : (x,y,z,w:A; h:(mor z w); g:(mor y z); f:(mor x y))
   (oo h (oo g f))==(oo (oo h g) f);
  ax_ii_sx : (x,y:A; f:(mor x y)) (oo (ii y) f)==f;
  ax_ii_dx : (x,y:A; f:(mor x y)) (oo f (ii x))==f
 }.

End Cat_Theory.

Record Cat_Data : Type := {
  obj :> Type;
  mor : obj->obj->Type;
  oo : (x,y,z:obj; g:(mor y z); f:(mor x y)) (mor x z);
  ii : (x:obj) (mor x x)
 }.

Implicits ii [1].

Record Cat : Type := {
  cat_data :> (Cat_Data);
  cat_theory :> (Cat_Theory (!oo cat_data) (!ii cat_data))
 }.

Section Cat_Facts.

Variable A:Cat.

Lemma oo_congr : (x,y,z:A; g1,g2:(mor y z); f1,f2:(mor x y))
 g1==g2 -> f1==f2 -> (oo g1 f1)==(oo g2 f2).
Proof.
  NewDestruct 1; NewDestruct 1; Reflexivity.
Qed.

Lemma oo_assoc : (x,y,z,w:A; h:(mor z w); g:(mor y z); f:(mor x y))
 (oo h (oo g f))==(oo (oo h g) f).
Proof.
  Apply (ax_oo_assoc A).
Qed.

Lemma ii_sx : (x,y:A; f:(mor x y))(oo (ii y) f)==f.
Proof.
  Apply (ax_ii_sx A).
Qed.

Lemma ii_dx : (x,y:A; f:(mor x y))(oo f (ii x))==f.
Proof.
  Apply (ax_ii_dx A).
Qed.

End Cat_Facts.

Hints Resolve oo_assoc ii_sx ii_dx oo_congr.

Definition Obj : Type := Cat.

Definition src [A:Cat; x,y:A; f:(mor x y)] : A := x.

Definition trg [A:Cat; x,y:A; f:(mor x y)] : A := y.

(** ** Equality between morphisms of a category *)

(** We introduce a notion of equality between morphisms that permits
    to compare morphisms with different source and targets. *)

Section Eq_Mor.

Variable A:Cat.

Inductive eq_mor [x,y:A; f:(mor x y)] : (u,v:A; g:(mor u v)) Prop :=
 | eq_mor_refl : (eq_mor f f).

Hints Resolve eq_mor_refl.

Lemma eqT_eq_mor : (x,y:A; f,g:(mor x y))
 f==g -> (eq_mor f g).
Proof.
  NewDestruct 1; Split.
Qed.

Lemma eq_mor_sym : (x,y:A; f:(mor x y); u,v:A; g:(mor u v))
 (eq_mor f g) -> (eq_mor g f).
Proof.
  NewDestruct 1; Split.
Qed.

Lemma eq_mor_trans :
 (x,y:A; f:(mor x y); u,v:A; g:(mor u v); s,t:A; h:(mor s t))
 (eq_mor f g) -> (eq_mor g h) -> (eq_mor f h).
Proof.
  NewDestruct 1; NewDestruct 1; Split.
Qed.

Lemma eq_mor_src : (x,y:A; f:(mor x y); u,v:A; g:(mor u v))
 (eq_mor f g) -> (src f) == (src g).
Proof.
  NewDestruct 1; Split.
Qed.

Lemma eq_mor_trg : (x,y:A; f:(mor x y); u,v:A; g:(mor u v))
 (eq_mor f g) -> (trg f) == (trg g).
Proof.
  NewDestruct 1; Split.
Qed.

(** Now we prove that this [eq_mor] actually specialize to [eqT] if
    the source and the target of the two morphisms being compared are
    the same. *)

Lemma eq_mor_eqT : (x,y:A; f,g:(mor x y)) (eq_mor f g) -> f==g.
Proof.
  Intros.
  LetTac P := [w,z:A; h:(mor w z)]
   (H:(mor w z)==(mor x y))f==(transport H h).
  Cut (P ? ? g).
  Unfold P; Intros.
  Apply (H0 (refl_eqT ? (mor x y))).
  Elim H.
  Unfold P; Intros.
  Rewrite transport_refl; Split.
Qed.

End Eq_Mor.

(** ** Inverse of a morphism. *)

Section Inverse_Mor.

Variable A:Cat; sx,dx:A; f:(mor sx dx).

Section Inverse_Mor_Theory.

Variable g:(mor dx sx).

Record Inverse_Mor_Theory : Prop := {
  ax_inverse_eq_sx : (oo g f)==(ii sx);
  ax_inverse_eq_dx : (oo f g)==(ii dx)
 }.

End Inverse_Mor_Theory.

Record Inverse_Mor : Type := {
  inverse_mor :> (mor dx sx);
  inverse_mor_theory : (Inverse_Mor_Theory inverse_mor)
 }.

Section Inverse_Mor_Facts.

Variable g:Inverse_Mor.

Lemma inverse_eq_sx : (oo g f)==(ii sx).
Proof.
  Elim g; Simpl; NewDestruct 1; Auto.
Qed.
  
Lemma inverse_eq_dx : (oo f g)==(ii dx).
Proof.
  Elim g; Simpl; NewDestruct 1; Auto.
Qed.

End Inverse_Mor_Facts.

Hints Resolve inverse_eq_sx inverse_eq_dx.

End Inverse_Mor.

End Cat.

(* ********************************************************************** *)

Module Type A_Cat.

Variable A:Cat.Obj.

End A_Cat.
