(* tag: Commutative rings *)

Require Export cat_sig.
Require Export monoid.
Require Export group.
Require Arith.

Implicit Arguments On.

Module CRing <: Cat_Sig.

Export Misc.

Section CRing_Theory.

Variable A:Type; add:A->A->A; zero:A; opp:A->A; mul:A->A->A; one:A.

Record CRing_Theory : Prop := {
  ax_add_assoc : (x,y,z:A) (add x (add y z))==(add (add x y) z);
  ax_add_commut : (x,y:A) (add x y)==(add y x);
  ax_zero_sx : (x:A) (add zero x)==x;
  ax_opp_sx : (x:A) (add (opp x) x)==zero;
  ax_mul_assoc : (x,y,z:A) (mul x (mul y z))==(mul (mul x y) z);
  ax_mul_commut : (x,y:A) (mul x y)==(mul y x);
  ax_one_sx : (x:A) (mul one x)==x;
  ax_distrib_sx : (x,y,z:A) (mul x (add y z))==(add (mul x y) (mul x z))
 }.

End CRing_Theory.

Record Ring_Data : Type := {
  ring_carrier :> Type;
  add : ring_carrier->ring_carrier->ring_carrier;
  zero : ring_carrier;
  opp : ring_carrier->ring_carrier;
  mul : ring_carrier->ring_carrier->ring_carrier;
  one : ring_carrier
 }.

Implicits zero [1].
Implicits one [1].

Record CRing : Type := {
  ring_data :> Ring_Data;
  ring_theory :> (CRing_Theory (!add ring_data) (!zero ring_data)
     (!opp ring_data) (!mul ring_data) (!one ring_data))
 }.

Definition Obj : Type := CRing.
Coercion CRing_of_Obj [x:Obj] : CRing := x.

Section CRing_Facts.

Variable A:CRing.

Lemma add_assoc : (x,y,z:A) (add x (add y z))==(add (add x y) z).
Proof.
  Intros; Apply (ax_add_assoc A).
Qed.

Lemma add_commut : (x,y:A) (add x y)==(add y x).
Proof.
  Intros; Apply (ax_add_commut A).
Qed.

Lemma zero_sx : (x:A) (add zero x)==x.
Proof.
  Intros; Apply (ax_zero_sx A).
Qed.

Lemma zero_dx : (x:A) (add x zero)==x.
Proof.
  Intros.
  Rewrite add_commut.
  Apply (ax_zero_sx A).
Qed.

Lemma opp_sx : (x:A) (add (opp x) x)==zero.
Proof.
  Intros; Apply (ax_opp_sx A).
Qed.

Lemma opp_dx : (x:A) (add x (opp x))==zero.
Proof.
  Intros.
  Rewrite add_commut.
  Apply (ax_opp_sx A).
Qed.

Lemma mul_assoc : (x,y,z:A) (mul x (mul y z))==(mul (mul x y) z).
Proof.
  Intros; Apply (ax_mul_assoc A).
Qed.

Lemma mul_commut : (x,y:A) (mul x y)==(mul y x).
Proof.
  Intros; Apply (ax_mul_commut A).
Qed.

Lemma one_sx : (x:A) (mul one x)==x.
Proof.
  Intros; Apply (ax_one_sx A).
Qed.

Lemma one_dx : (x:A) (mul x one)==x.
Proof.
  Intros.
  Rewrite mul_commut.
  Apply (ax_one_sx A).
Qed.

Lemma distrib_sx : (x,y,z:A)
 (mul x (add y z))==(add (mul x y) (mul x z)).
Proof.
  Intros; Apply (ax_distrib_sx A).
Qed.

Lemma distrib_dx : (x,y,z:A)
 (mul (add y z) x)==(add (mul y x) (mul z x)).
Proof.
  Intros.
  Rewrite mul_commut with (add y z) x.
  Rewrite (ax_distrib_sx A).
  Apply plug2; Apply mul_commut.
Qed.

Lemma add_sx_cancel : (x,y,z:A)
 (add x y)==(add x z) -> y==z.
Proof.
  Intros.
  Rewrite <- zero_sx with y.
  Rewrite <- opp_sx with x.
  Rewrite <- add_assoc.
  Rewrite H.
  Rewrite add_assoc.
  Rewrite opp_sx.
  Apply zero_sx.
Qed.

Lemma add_dx_cancel : (x,y,z:A)
 (add y x)==(add z x) -> y==z.
Proof.
  Intros.
  Apply add_sx_cancel with x.
  Rewrite add_commut with x y.
  Rewrite H.
  Apply add_commut.
Qed.

Lemma opp_sx_char : (x,y:A)
  (add x y)==zero -> (opp y)==x.
Proof.
  Intros.
  Apply add_dx_cancel with y.
  Rewrite H.
  Apply opp_sx.
Qed.
  
Lemma opp_dx_char : (x,y:A)
  (add x y)==zero -> (opp x)==y.
Proof.
  Intros.
  Apply add_sx_cancel with x.
  Rewrite H.
  Apply opp_dx.
Qed.
  
Lemma opp_add : (x,y:A)
 (opp (add x y))==(add (opp x) (opp y)).
Proof.
  Intros.
  Rewrite add_commut with x y.
  Apply opp_sx_char.
  Rewrite add_assoc.
  Rewrite <- add_assoc with (opp x) (opp y) y.
  Rewrite opp_sx.
  Rewrite zero_dx.
  Apply opp_sx.
Qed.

Lemma opp_opp : (x:A)(opp (opp x))==x.
Proof.
  Intros; Apply opp_sx_char.
  Apply opp_dx.
Qed.

Lemma opp_zero : (opp (!zero A))==zero.
Proof.
  Apply opp_sx_char.
  Apply zero_sx.
Qed.

Lemma mul_sx_zero : (x:A) (mul zero x)==zero.
Proof.
  Intros.
  Apply add_sx_cancel with (mul zero x).
  Rewrite zero_dx.
  Rewrite <- distrib_dx.
  Rewrite zero_sx.
  Reflexivity.
Qed.

Lemma mul_dx_zero : (x:A) (mul x zero)==zero.
Proof.
  Intros.
  Apply add_sx_cancel with (mul x zero).
  Rewrite zero_dx.
  Rewrite <- distrib_sx.
  Rewrite zero_sx.
  Reflexivity.
Qed.

Lemma opp_mul_sx : (x,y:A) (opp (mul x y))==(mul (opp x) y).
Proof.
  Intros.
  Apply opp_sx_char.
  Rewrite <- distrib_dx.
  Rewrite opp_sx.
  Apply mul_sx_zero.
Qed.

Lemma opp_mul_dx : (x,y:A) (opp (mul x y))==(mul x (opp y)).
Proof.
  Intros.
  Apply opp_sx_char.
  Rewrite <- distrib_sx.
  Rewrite opp_sx.
  Apply mul_dx_zero.
Qed.

End CRing_Facts.

Hints Resolve add_assoc add_commut zero_sx zero_dx opp_sx opp_dx
mul_assoc mul_commut one_sx one_dx distrib_sx distrib_dx.

Fixpoint power [A:CRing; x:A; n:nat] : A :=
  Cases n of
    | O => one
    | (S n) => (mul x (power x n))
  end.

Lemma power_add : (A:CRing; x:A; n,m:nat)
  (mul (power x n) (power x m))==(power x (plus n m)).
Proof.
  Simpl; Intros.
  NewInduction n; NewInduction m; Simpl; Auto.
  Transitivity (mul x (power x n)).
  Auto.
  Replace (plus n O) with n; Auto with arith.
  Rewrite <- mul_assoc; Auto.
Qed.

Lemma power_mul : (A:CRing; x,y:A; n:nat)
  (power (mul x y) n)==(mul (power x n) (power y n)).
Proof.
  Simpl; Intros.
  NewInduction n; Simpl; Auto.
  Rewrite IHn.
  Repeat Rewrite <- mul_assoc.
  Apply plug2; Trivial.
  Repeat Rewrite mul_assoc; Auto.
Qed.

Lemma power_one : (A:CRing; n:nat)
 (power one n)==(!one A).
Proof.
  NewInduction n; Simpl.
  Auto.
  Rewrite IHn.
  Auto.
Qed.

Lemma power_power : (A:CRing; x:A; n,m:nat)
  (power (power x n) m)==(power x (mult n m)).
Proof.
  Intros.
  NewInduction m; Simpl.

  Replace (mult n O) with O; Auto with arith.

  Rewrite IHm.
  Replace (mult n (S m)) with (plus n (mult n m)).
  Rewrite <- power_add; Auto.
  Transitivity (plus (mult n m) n); Auto with arith.
Qed.

Record Morph_Theory [A,B:CRing; f:A->B] : Prop := {
  morph_add : (x,y:A) (f (add x y))==(add (f x) (f y));
  morph_opp : (x:A) (f (opp x))==(opp (f x));
  morph_mul : (x,y:A) (f (mul x y))==(mul (f x) (f y));
  morph_one : (f one)==one
 }.

Record CRing_Morph [A,B:CRing] : Type := {
  morph_data :> A->B;
  morph_theory :> (Morph_Theory morph_data)
 }.

Definition Morph := CRing_Morph.

Lemma morph_extens : (A,B:CRing; f,g:(Morph A B))
 ((x:A)(f x)==(g x)) -> f==g.
Proof.
  NewDestruct f; NewDestruct g; Simpl; Intros.
  Assert U : morph_data0 == morph_data1.
  Apply nondep_extensionality; Assumption.
  NewDestruct U.
  Assert U : morph_theory0 == morph_theory1.
  Apply proof_irrelevance.
  NewDestruct U.
  Reflexivity.
Qed.

Section morph_comp.

Variable A,B,C:CRing.

Variable f:(Morph B C); g:(Morph A B).

Local h [x:A] : C := (f (g x)).

Remark h_th : (Morph_Theory h).
Proof.
  Unfold h; Split; Intros.
  Rewrite (morph_add g).
  Rewrite (morph_add f).
  Reflexivity.
  Rewrite (morph_opp g).
  Rewrite (morph_opp f).
  Reflexivity.
  Rewrite (morph_mul g).
  Rewrite (morph_mul f).
  Reflexivity.
  Rewrite (morph_one g).
  Rewrite (morph_one f).
  Reflexivity.
Qed.

Definition morph_comp : (Morph A C).
Proof.
  Split with h;
  Apply h_th.
Defined.

End morph_comp.

Definition morph_id [A:CRing] : (Morph A A).
Proof.
  Intros.
  Split with [x:A]x.
  Abstract Split; Auto.
Defined.

Implicits morph_id [1].

Lemma morph_comp_assoc : (A,B,C,D:CRing)
 (h:(Morph C D); g:(Morph B C); f:(Morph A B))
 (morph_comp h (morph_comp g f))==(morph_comp (morph_comp h g) f).
Proof.
  Intros; Apply morph_extens; Trivial.
Qed.

Lemma morph_id_sx : (A,B:CRing; f:(Morph A B))
 (morph_comp morph_id f)==f.
Proof.
  Intros; Apply morph_extens; Trivial.
Qed.

Lemma morph_id_dx : (A,B:CRing; f:(Morph A B))
 (morph_comp f morph_id)==f.
Proof.
  Intros; Apply morph_extens; Trivial.
Qed.

End CRing.
