(* tag: diagrams *)

Require Export graph_.

Implicit Arguments On.

(** * Diagrams. *)

(**
  A "diagram" [D] in a category [A] is a [Graph] with two maps:
   - [diagram] which map nodes of [D] to objects of [A];
   - [diagram_arc] which maps arcs of [D] to morphisms in [A].
  
  A "commutative" diagram is a diagram with the following property:
  if two arcs have the same source and target, their associated
  morphisms are equal.
  
  The type of all diagrams (resp. commutative diagram) in a category
  [A] is [(Diagram A)] (resp. [(CDiagram A)]).
  
  We call [Diagram_Functor] the functor associated to a diagram (of
  type [(Functor (Cat_of_Graph D) A)]).
 *)

Module Diagram_.

Export Graph_.

Section Diagram.

Variable A:Cat.

(** Definition of diagram. *)
Record Diagram : Type := {
  diagram_graph :> Graph;
  diagram : diagram_graph -> A;
  diagram_arc : (f:(arc diagram_graph))
   (mor (diagram (arc_src f)) (diagram (arc_trg f)))
 }.

(** Definition of commutagive diagram. *)
Definition CDiagram_Theory [D:Diagram] : Prop :=
 (f,g:(arc D))
 (arc_src f)==(arc_src g) -> (arc_trg f)==(arc_trg g) ->
 (eq_mor (diagram_arc f) (diagram_arc g)).

Record CDiagram : Type := {
  cdiagram_data :> Diagram;
  ax_cdiagram_commut : (CDiagram_Theory cdiagram_data)
 }.

Lemma cdiagram_commut : (D:CDiagram)
 (f,g:(arc D))
 (arc_src f)==(arc_src g) -> (arc_trg f)==(arc_trg g) ->
 (eq_mor (diagram_arc f) (diagram_arc g)).
Proof.
  NewDestruct D; Trivial.
Qed.

Hints Resolve cdiagram_commut.

(** We build the Functor associated to the diagram [D]. *)
Section Diagram_Functor.

Variable D:Diagram.

(** Map from arrows of [D] to morphisms of [A]. *)
Definition diagram_arrow [x,y:D; f:(Arrow x y)] :
 (mor (diagram x) (diagram y)).
Proof.
  Intros.
  Rewrite <- (arrow_trg_eq f).
  Pattern 1 x.
  Rewrite <- (arrow_src_eq f).
  Simpl.
  Apply diagram_arc.
Defined.

(** Map from paths of [D] to morphisms of [A]. *)
Definition diagram_path [x,y:D; f:(Path x y)] :
 (mor (diagram x) (diagram y)).
Proof.
  NewInduction 1.
  Apply ii.
  Apply oo with (diagram y).
  Exact (diagram_arrow u).
  Apply IHf.
Defined.

Local G : (Functor_Data (Cat_of_Graph D) A).
Proof.
  Split with (!diagram D).
  Exact diagram_path.
Defined.

Remark diagram_path_comp : (x,y,z:D; g:(Path y z); f:(Path x y))
 (diagram_path (path_oo g f))==(oo (diagram_path g) (diagram_path f)).
Proof.
  NewInduction g; Simpl; Intros.
  Rewrite ii_sx.
  Reflexivity.
  Rewrite IHg.
  Rewrite <- oo_assoc.
  Reflexivity.
Qed.

Remark HG : (Functor_Theory (fct G)).
Proof.
  Split; Simpl; Intros.
  Apply diagram_path_comp.
  Reflexivity.
Qed.

Definition Diagram_Functor : (Functor (Cat_of_Graph D) A).
Proof.
  Split with G; Exact HG.
Defined.

End Diagram_Functor.

End Diagram.

End Diagram_.
