(* tag: Equivalence relations *)

(** * Equivalence relations *)

Require Export misc_.

Implicit Arguments On.

Module Equiv_.

Export Misc_.

Section Equivalence_Theory.

Variable A:Type; Q:A->A->Prop.

Record Equivalence_Theory : Prop := {
  ax_eqv_refl : (x:A) (Q x x);
  ax_eqv_sym : (x,y:A) (Q x y) -> (Q y x);
  ax_eqv_trans : (x,y,z:A) (Q x y) -> (Q y z) -> (Q x z)
 }.

End Equivalence_Theory.

Record Equivalence [A:Type] : Type := {
  equivalence_pred :> A->A->Prop;
  equivalence_theory :> (Equivalence_Theory equivalence_pred)
 }.

Section Equivalence_Facts.

Variable A:Type; Q:(Equivalence A).

Lemma eqv_refl : (x:A) (Q x x).
Proof.
  Intros; Apply (ax_eqv_refl Q).
Qed.

Lemma eqv_eqT : (x,y:A) x==y -> (Q x y).
Proof.
  NewDestruct 1; Apply eqv_refl.
Qed.

Lemma eqv_sym : (x,y:A) (Q x y) -> (Q y x).
Proof.
  Intros; Apply (ax_eqv_sym Q); Assumption.
Qed.

Lemma eqv_trans : (x,y,z:A) (Q x y) -> (Q y z) -> (Q x z).
Proof.
  Intros; Apply (ax_eqv_trans Q) with y; Assumption.
Qed.

End Equivalence_Facts.

Hints Resolve eqv_refl eqv_eqT eqv_sym eqv_trans.

Definition Compatible [X:Type; Q:(Equivalence X); Y:Type; f:X->Y] :=
  (x,y:X) (Q x y) -> (f x)==(f y).
Hints Unfold Compatible.

(** Equivalence relation generated by a binary relation *)

Section Eqgen.

Variable A:Type; P:A->A->Prop.

Inductive eqgen_pred : A->A->Prop :=
 | ax_eqgen_intro : (x,y:A) (P x y) -> (eqgen_pred x y)
 | ax_eqgen_ref : (x:A) (eqgen_pred x x)
 | ax_eqgen_sym : (x,y:A) (eqgen_pred x y) -> (eqgen_pred y x)
 | ax_eqgen_trs : (x,y,z:A)
    (eqgen_pred x y) -> (eqgen_pred y z) -> (eqgen_pred x z).

Hints Resolve ax_eqgen_intro ax_eqgen_ref ax_eqgen_sym ax_eqgen_trs.

Definition eqgen : (Equiv_.Equivalence A).
Proof.
  Split with eqgen_pred.
  Abstract Split; EAuto.
Defined.

Lemma eqgen_intro : (x,y:A) (P x y) -> (eqgen x y).
Proof.
Simpl; Auto.
Qed.

(* !!!
Lemma eqgen_ref : (x:A) (eqgen x x).
Proof.
Simpl; Auto.
Qed.

Lemma eqgen_sym : (x,y:A) (eqgen_pred x y) -> (eqgen_pred y x).
Proof.
Simpl; Auto.
Qed.

Lemma eqgen_trs : (x,y,z:A)
    (eqgen_pred x y) -> (eqgen_pred y z) -> (eqgen_pred x z).
Proof.
Simpl; EAuto.
Qed.

Hints Resolve eqgen_intro eqgen_ref eqgen_sym eqgen_trs.
*)
(* !!!
Lemma eqgen_equivalence : (Equivalence_Theory P) ->
 ([x,y:A](eqgen x y))==P.
Proof.
  Intros.
  Apply nondep_extensionality; Intro.
  Apply nondep_extensionality; Intro.
  Apply prop_uni.
  NewDestruct H; NewInduction 1; EAuto.
  Auto.
Qed.
*)

(** A function which is compatible with a binary relation
    is also compatible for the generated equivalence
    relation. *)
Lemma eqgen_compat : (B:Type; f:A->B)
 ((x,y:A)(P x y) -> (f x)==(f y)) ->
 (Compatible eqgen f).
Proof.
  Intros; Intro; Intros.
  NewInduction H0; Auto.
  (* !!! Transitivity *)
  Apply trans_eqT with (f y); Auto.
Qed.

End Eqgen.

(* !!!
Hints Resolve eqgen_intro eqgen_ref eqgen_sym eqgen_trs.
*)
Hints Resolve eqgen_intro.

End Equiv_.
