(* tag: functors *)

Require Export cat___.
Require cat_sig___.
Require misc___.
Require map___.

(** * Functors *)

Implicit Arguments On.

Module Functor___ <: Cat_Sig___.

Export Cat___.
Import Misc___.
Import Map___.

Section Functor_Theory.

Variable A,B:Cat.
Variable F0:A->B; F1:(x,y:A) (mor x y) -> (mor (F0 x) (F0 y)).

Record Functor_Theory : Prop := {
  ax_fct_oo : (x,y,z:A; g:(mor y z); f:(mor x y))
   (F1 (oo g f))==(oo (F1 g) (F1 f));
  ax_fct_ii : (x:A) (F1 (ii x))==(ii (F0 x))
 }.

End Functor_Theory.

Record Functor_Data [A,B:Cat] : Type := {
  fct_obj :> A->B;
  fct : (x,y:A; f:(mor x y)) (mor (fct_obj x) (fct_obj y))
 }.

Record Functor [A,B:Cat] : Type := {
  functor_data :> (Functor_Data A B);
  functor_theory :> (Functor_Theory (fct functor_data))
 }.

Section Functor_Facts.

Variable A,B:Cat; F:(Functor A B).

Lemma fct_oo : (x,y,z:A; g:(mor y z); f:(mor x y))
 (fct F (oo g f))==(oo (fct F g) (fct F f)).
Proof.
  Intros; Apply (ax_fct_oo F).
Qed.

Lemma fct_ii : (x:A) (fct F (ii x))==(ii (F x)).
Proof.
  Intros; Apply (ax_fct_ii F).
Qed.

End Functor_Facts.

Hints Resolve fct_oo fct_ii.

Definition Morph [A,B:Cat] : Type := (Functor A B).

Section Eq_Fct.

Variable A,B:Cat.

Definition eq_fct [F,G:(Functor A B)] : Prop :=
 (x,y:A; f:(mor x y)) (eq_mor (fct F f) (fct G f)).

Variable F,G:(Functor A B); F_eq_G:(eq_fct F G).

Lemma eq_fct_src : (x,y:A; f:(mor x y))
 (src (fct F f))==(src (fct G f)).
Proof.
  Intros; NewDestruct (F_eq_G f); Reflexivity.
Qed.

Lemma eq_fct_trg : (x,y:A; f:(mor x y))
 (trg (fct F f))==(trg (fct G f)).
Proof.
  Intros; NewDestruct (F_eq_G f); Reflexivity.
Qed.

Lemma eq_fct_obj : (x:A)(F x)==(G x).
Proof.
  Intros.
  Change (src (ii (F x)))==(src (ii (G x))).
  Do 2 Rewrite <- fct_ii.
  Apply eq_fct_src.
Qed.

(** ** Extensionality for functors *)

(** We prove that [eq_fct] implies [eqT]. *)

Lemma functor_extens : F==G.
Proof.
  Assert U := eq_fct_obj.
  NewDestruct F; NewDestruct G.
  NewDestruct functor_data0.
  NewDestruct functor_data1.
  Simpl in U.
  Assert E : fct_obj0==fct_obj1.
  Apply nondep_extensionality.
  Apply U.
  NewDestruct E.
  Clear F G U.
  Rename fct_obj0 into FF.
  Assert E : fct0==fct1.
  Apply extensionality
  with T := [x:A](y:A)(mor x y)->(mor (FF x) (FF y)).
  Intros.
  Apply extensionality
  with T := [y:A](mor x y)->(mor (FF x) (FF y)).
  Intros.
  Apply nondep_extensionality.
  Intros.
  Unfold eq_fct in F_eq_G.
  Simpl in F_eq_G.
  Apply eq_mor_eqT.
  Apply (F_eq_G x1).
  NewDestruct E.
  Assert E : functor_theory0 == functor_theory1.
  Apply proof_irrelevance.
  NewDestruct E.
  Reflexivity.
Qed.

End Eq_Fct.

(** ** Category of categories. *)

(** Now we prove that categories with functors form a category (in a
    higher universe).  We do not build the term of type [Cat____.Obj] in
    this file.  We will use the parametric module
    [Make_Small_Cat___] to do that.  Here we simply make sure that the
    module [Cat___] will be of module type [Cat_Sig___] by defying the
    term [morph_id], [morph_comp] etc.
  *)

(** Identity functor. *)

Definition morph_id [A:Cat] : (Functor A A).
Proof.
  Intro.
  Split with (!Build_Functor_Data A A [x:A]x [x,y:A; f:(mor x y)]f).
  Abstract Split; Simpl; Intros; Reflexivity.
Defined.

(** Composition of functors. *)

Section Functor_Comp.

Variable A,B,C:Cat.
Variable G:(Functor B C); F:(Functor A B).

Local h : (Functor_Data A C) :=
(!Build_Functor_Data A C
  [x:A](G (F x))
  [x,y:A; f:(mor x y)] (fct G (fct F f))).

Remark h_th : (Functor_Theory (fct h)).
Proof.
  Split; Simpl; Intros.
  Repeat Rewrite fct_oo; Auto.
  Repeat Rewrite fct_ii; Auto.
Qed.

Definition morph_comp : (Functor A C).
Proof.
  Split with h; Exact h_th.
Defined.
End Functor_Comp.

Lemma morph_comp_assoc : (A,B,C,D:Cat)
 (H:(Functor C D); G:(Functor B C); F:(Functor A B))
 (morph_comp H (morph_comp G F))==(morph_comp (morph_comp H G) F).
Proof.
  Intros; Apply functor_extens.
  Unfold morph_comp eq_fct; Simpl; Intros.
  Apply eq_mor_refl.
Qed.

Lemma morph_id_sx : (A,B:Cat; F:(Functor A B))
 (morph_comp (morph_id B) F)==F.
Proof.
  Intros; Apply functor_extens.
  Unfold morph_comp morph_id eq_fct; Simpl; Intros.
  Apply eq_mor_refl.
Qed.

Lemma morph_id_dx : (A,B:Cat; F:(Functor A B))
 (morph_comp F (morph_id A))==F.
Proof.
  Intros; Apply functor_extens.
  Unfold morph_comp morph_id eq_fct; Simpl; Intros.
  Apply eq_mor_refl.
Qed.

Section Embedding.

Variable A,B:Cat; F:(Functor A B).

Definition embedding : Type := (x,y:A)
 (inverse [f:(mor x y)](fct F f)).

End Embedding.

Definition Obj := Cat___.Cat.

End Functor___.
