(* tag: graphs *)

Require functor.

Implicit Arguments On.

(** * Graphs *)

Module Graph.

Export Functor.

(** ** Definition of Graph *)

Section Graph.

Record Graph : Type := {
  node :> Type;
  arc : Type;
  arc_src : (f:arc) node;
  arc_trg : (f:arc) node
 }.

Variable G:Graph.

(** ** Arrows. *)

(** [(Arrow x y)] is the type of the arcs with endpoints [x], [y]. *)

Section Arrow_Theory.

Variable f:(arc G).

Record Arrow_Theory [x,y:G] : Type := {
  ax_arrow_src_eq : (arc_src f)==x;
  ax_arrow_trg_eq : (arc_trg f)==y
 }.

End Arrow_Theory.

Record Arrow [x,y:G] : Type := {
  arrow_data :> (arc G);
  arrow_theory : (Arrow_Theory arrow_data x y)
 }.

Section Arrow_Facts.

Variable x,y:G; f:(Arrow x y).

Local Hf := (arrow_theory f).

Lemma arrow_src_eq : (arc_src f)==x.
Proof.
  NewDestruct Hf; Auto.
Qed.

Lemma arrow_trg_eq : (arc_trg f)==y.
Proof.
  NewDestruct Hf; Auto.
Qed.

End Arrow_Facts.

Hints Resolve arrow_src_eq arrow_trg_eq.

Definition make_arrow [f:(arc G)] : (Arrow (arc_src f) (arc_trg f)).
Proof.
  Intros; Split with f.
  Abstract Split; Reflexivity.
Defined.

(** ** Paths *)

(** Paths are chains of arrows. *)
Inductive Path : (x,y:G) Type :=
 | path_id : (x:G) (Path x x)
 | path_cons : (y,z:G; u:(Arrow y z); x:G; f:(Path x y))
    (Path x z).

(** Composition of paths. *)
Definition path_oo [x,y,z:G; g:(Path y z); f:(Path x y)] : (Path x z).
Proof.
  Intros.
  NewInduction g.
  Intros.
  Exact f.
  Apply path_cons with y.
  Exact u.
  Apply IHg.
  Exact f.
Defined.

(** Consing and composition. *)
Remark path_oo_cons :
 (z,w:G; u:(Arrow z w); y:G; g:(Path y z); x:G; f:(Path x y))
 (path_oo (path_cons u g) f)==(path_cons u (path_oo g f)).
Proof.
  Reflexivity.
Qed.

(** Associativity of the composition. *) 
Remark path_oo_assoc :
 (z,w:G; h:(Path z w); y:G; g:(Path y z); x:G; f:(Path x y))
 (path_oo h (path_oo g f))==(path_oo (path_oo h g) f).
Proof.
  NewInduction h.
  Reflexivity.
  NewDestruct g.
  Intros.
  Repeat Rewrite path_oo_cons.
  Rewrite IHh.
  Reflexivity.

  NewDestruct f.
  Intros.
  Repeat Rewrite path_oo_cons.
  Repeat Rewrite <- IHh.
  Reflexivity.

  Intros.
  Repeat Rewrite path_oo_cons.
  Rewrite <- IHh.
  Reflexivity.
Qed.

(** Right identity of the composition. *) 
Remark path_id_dx : (x,y:G; f:(Path x y))
 (path_oo f (path_id x))==f.
Proof.
  NewInduction f.
  Reflexivity.
  Rewrite path_oo_cons.
  Rewrite IHf.
  Reflexivity.
Qed.

(** * Category generated by a graph. *)

(** Category where with objects given by node and arrows given by
    paths of a give graph. *)

Section Cat_of_Graph.

Remark cat_of_graph_theory : (Cat_Theory path_oo path_id).
Proof.
  Split.
  Intros; Apply path_oo_assoc.
  Reflexivity.
  Apply path_id_dx.
Qed.

Canonical Structure Cat_of_Graph : Cat.
Proof.
  Split with (Build_Cat_Data path_oo path_id).
  Exact cat_of_graph_theory.
Defined.

End Cat_of_Graph.

End Graph.

(* !!!
Definition Diagram [A:Cat] : Type := (Functor Cat_of_Graph A).
*)

(** ** Morphisms of Graphs *)

Section Graph_Morph.

Variable G,H:Graph.

Section Morph_Theory.

Variable F0:G->H; F1:(arc G)->(arc H).

Record Graph_Morph_Theory : Prop := {
  ax_graph_morph_src_eq : (f:(arc G)) (arc_src (F1 f))==(F0 (arc_src f));
  ax_graph_morph_trg_eq : (f:(arc G)) (arc_trg (F1 f))==(F0 (arc_trg f))
 }.

End Morph_Theory.

Record Graph_Morph : Type := {
  graph_morph_carrier :> G->H;
  graph_morph : (arc G)->(arc H);
  graph_morph_theory : (Graph_Morph_Theory graph_morph_carrier graph_morph)
 }.

Section Graph_Morph_Facts.

Variable F:Graph_Morph.

Local HF := (graph_morph_theory F).

(** A morphism of graph preserve source and target of arcs. *)
Lemma graph_morph_src_eq : (f:(arc G))
 (arc_src (graph_morph F f))==(F (arc_src f)).
Proof.
  NewDestruct HF; Assumption.
Qed.

Lemma graph_morph_trg_eq : (f:(arc G))
 (arc_trg (graph_morph F f))==(F (arc_trg f)).
Proof.
  NewDestruct HF; Assumption.
Qed.

Hints Resolve graph_morph_src_eq graph_morph_trg_eq.

(** Action of a morphism of graph on arrows. *)

Section Graph_Morph_Arrow.

Remark graph_morph_arrow_theory : (x,y:G; u:(Arrow x y))
 (Arrow_Theory (graph_morph F u) (F x) (F y)).
Proof.
  Split.
  Rewrite graph_morph_src_eq.
  Rewrite arrow_src_eq.
  Reflexivity.
  Rewrite graph_morph_trg_eq.
  Rewrite arrow_trg_eq.
  Reflexivity.
Qed.  

Definition graph_morph_arrow : (x,y:G; u:(Arrow x y)) (Arrow (F x) (F y)).
Proof.
  Intros; Split with (graph_morph F u).
  Apply graph_morph_arrow_theory.
Defined.

End Graph_Morph_Arrow.

(** Action of a morphism of graph on paths. *)
Definition graph_morph_path [x,y:G; f:(Path x y)] : (Path (F x) (F y)).
Proof.
  NewInduction 1.
  Apply path_id.
  Apply path_cons with (F y).
  Split with (graph_morph F u).
  Apply graph_morph_arrow_theory.
  Exact IHf.
Defined.

(** Morphism of graphs respect path composition. *)
Lemma graph_morph_path_oo : (x,y,z:G; g:(Path y z); f:(Path x y))
 (graph_morph_path (path_oo g f))
  == (path_oo (graph_morph_path g) (graph_morph_path f)).
Proof.
  NewInduction g.
  Reflexivity.
  Simpl.
  Intros.
  Rewrite IHg.
  Reflexivity.
Qed.

(** ** Functor associated to a morphism of graphs. *)

(** Each morphism of graphs induce a functor induced on the
    corrisponding generated category. *)

Section Graph_Morph_Functor.

Local graph_morph_functor_data :
 (Functor_Data (Cat_of_Graph G) (Cat_of_Graph H)).
Proof.
  Split with [x:(Cat_of_Graph G)](F x).
  Exact graph_morph_path.
Defined.

Remark graph_morph_functor_theory :
 (Functor_Theory (fct graph_morph_functor_data)).
Proof.
  Split; Simpl; Intros.
  Apply graph_morph_path_oo.
  Reflexivity.
Qed.

Definition Graph_Morph_Functor : (Functor (Cat_of_Graph G) (Cat_of_Graph H)).
Proof.
  Split with graph_morph_functor_data.
  Exact graph_morph_functor_theory.
Defined.

End Graph_Morph_Functor.

End Graph_Morph_Facts.

End Graph_Morph.

End Graph.
