(* tag: Ideals. *)

Require Export cring.

Module Ideal.

Export CRing.

Section Ideal.

Variable A:CRing.

Section Ideal_Theory.

Variable P : (x:A)Prop.

Record Ideal_Theory : Prop := {
  ax_ideal_zero : (P zero);
  ax_ideal_add : (x,y:A) (P x) -> (P y) -> (P (add x y));
  ax_ideal_mul : (a,x:A) (P x) -> (P (mul a x))
 }.

End Ideal_Theory.

Record Ideal : Type := {
  ideal_pred :> (x:A) Prop;
  ideal_theory : (Ideal_Theory ideal_pred)
 }.

Section Ideal_Facts.

Variable I : Ideal.
Local HI := (ideal_theory I).

Lemma ideal_zero : (I zero).
Proof.
  NewDestruct HI; Auto.
Qed.

Lemma ideal_add : (x,y:A)
 (I x) -> (I y) -> (I (add x y)).
Proof.
  NewDestruct HI; Auto.
Qed.

Lemma ideal_mul : (a,x:A) (I x) -> (I (mul a x)).
Proof.
  NewDestruct HI; Auto.
Qed.

Hints Resolve ideal_zero ideal_add ideal_mul.

Lemma ideal_opp : (x:A) (I x) -> (I (opp x)).
Proof.
  Intros.
  Replace (opp x) with (mul (opp one) x).
  Auto.
  Rewrite <- opp_mul_sx; Auto.
Qed.

Hints Resolve ideal_opp.

End Ideal_Facts.

Hints Resolve ideal_zero ideal_add ideal_mul ideal_opp.

Definition ideal_sub [I,J:Ideal] : Prop :=
 (x:A) (I x) -> (J x).

Definition ideal_eq [I,J:Ideal] : Prop :=
 (ideal_sub I J) /\ (ideal_sub J I).

Lemma ideal_extens : (I,J:Ideal)
 (ideal_eq I J) -> I==J.
Proof.
  NewDestruct I; NewDestruct J.
  Unfold ideal_eq ideal_sub; Simpl.
  NewDestruct 1.
  Assert U : ideal_pred0==ideal_pred1.
  Apply nondep_extensionality; Intros.
  Apply prop_uni; Auto.
  NewDestruct U.
  Assert U : ideal_theory0==ideal_theory1.
  Apply proof_irrelevance.
  NewDestruct U.
  Reflexivity.
Qed.

Definition Trivial_Ideal : Ideal.
Proof.
  Split with [x:A] True.
  Abstract Split; Split.
Defined.

Section Zero_Ideal.

Inductive Zero_Ideal_Pred : (x:A) Prop :=
 | zero_ideal_intro : (Zero_Ideal_Pred zero).

Remark zero_ideal_theory : (Ideal_Theory Zero_Ideal_Pred).
Proof.
  Split.
  Split.
  NewDestruct 1; NewDestruct 1.
  Rewrite zero_sx. 
  Split.
  NewDestruct 1.
  Rewrite mul_dx_zero.
  Split.
Qed.

Definition Zero_Ideal : Ideal.
Proof.
  Split with Zero_Ideal_Pred.
  Exact zero_ideal_theory.
Defined.

End Zero_Ideal.

Section Cap_Ideal.

Variable I,J:Ideal.

Local Cap_Ideal_Pred [x:A] : Prop := (I x) /\ (J x).

Remark cup_ideal_theory : (Ideal_Theory Cap_Ideal_Pred).
Proof.
  Split.
  Split; Auto.
  NewDestruct 1; NewDestruct 1.
  Split; Auto.
  NewDestruct 1.
  Split; Auto.
Qed.

Definition Cap_Ideal : Ideal.
Proof.
  Split with Cap_Ideal_Pred.
  Exact cup_ideal_theory.
Defined.

End Cap_Ideal.

Section Add_Ideal.

Variable I,J:Ideal.

Inductive Add_Ideal_Pred : (x:A)Prop :=
 | add_ideal_intro : (x,y:A) (I x) -> (J y) -> (Add_Ideal_Pred (add x y)).

Lemma add_ideal_intro2 : (x,y,z:A)
 (I x) -> (J y) -> z==(add x y) -> (Add_Ideal_Pred z).
Proof.
  Intros.
  Rewrite H1.
  Split; Auto.
Qed.

Remark add_ideal_theory : (Ideal_Theory Add_Ideal_Pred).
Proof.
  Split.
  Apply add_ideal_intro2 with (!zero A) (!zero A); Auto.
  NewDestruct 1; NewDestruct 1.
  Replace (add (add x y0) (add x0 y)) with (add (add x x0) (add y0 y)).
  Split; Auto.
  Repeat Rewrite add_assoc.
  Apply plug2; Try Reflexivity.
  Repeat Rewrite <- add_assoc.
  Apply plug2; Auto.
  NewDestruct 1.
  Rewrite distrib_sx.
  Split; Auto.
Qed.

Definition Add_Ideal : Ideal.
Proof.
  Split with Add_Ideal_Pred.
  Exact add_ideal_theory.
Defined.

End Add_Ideal.

Section Generated_Ideal.

Variable P : (x:A) Prop.

Inductive Ideal_Gen_Pred : (x:A) Prop :=
 | ideal_gen_intro : (x:A) (P x) -> (Ideal_Gen_Pred x)
 | ideal_gen_zero : (Ideal_Gen_Pred zero)
 | ideal_gen_add : (x,y:A) (Ideal_Gen_Pred x) -> (Ideal_Gen_Pred y) ->
    (Ideal_Gen_Pred (add x y))
 | ideal_gen_mul : (a,x:A) (Ideal_Gen_Pred x) -> (Ideal_Gen_Pred (mul a x)).

Hints Resolve ideal_gen_intro ideal_gen_zero ideal_gen_add ideal_gen_mul.

Remark ideal_gen_theory : (Ideal_Theory Ideal_Gen_Pred).
Proof.
  Split.
  Apply ideal_gen_zero.
  NewDestruct 1; NewDestruct 1; Auto.
  NewDestruct 1; Auto.
Qed.

Definition Ideal_Gen : Ideal.
Proof.
  Split with Ideal_Gen_Pred.
  Exact ideal_gen_theory.
Defined.

Lemma ideal_gen_sub : (I:Ideal; HI:(x:A)(P x) -> (I x))
 (ideal_sub Ideal_Gen I).
Proof.
  Unfold ideal_sub; NewInduction 2; Auto.
Qed.

End Generated_Ideal.

Lemma ideal_gen_eq : (I:Ideal) I==(Ideal_Gen I).
Proof.
  Intros; Apply ideal_extens.
  Split.
  Unfold ideal_sub; Intros; Simpl. 
  Apply ideal_gen_intro; Assumption.
  Apply ideal_gen_sub.
  Tauto.
Qed.

Section Mul_Ideal.

Variable I,J:Ideal.

Inductive Mul_Ideal_Pred : (x:A) Prop :=
 | mul_ideal_intro : (x,y:A) (I x) -> (J y) -> (Mul_Ideal_Pred (mul x y))
 | mul_ideal_add : (x,y:A) (Mul_Ideal_Pred x) -> (Mul_Ideal_Pred y) ->
    (Mul_Ideal_Pred (add x y)).

Hints Resolve mul_ideal_intro mul_ideal_add.

Remark mul_ideal_theory : (Ideal_Theory Mul_Ideal_Pred).
Proof.
  Split.
  Replace (!zero A) with (!mul A zero zero).
  Auto.
  Apply mul_sx_zero.
  NewDestruct 1; NewDestruct 1; Auto.
  NewInduction 1.
  Rewrite mul_assoc.
  Auto.
  Rewrite distrib_sx.
  Auto.
Qed.

Definition Mul_Ideal : Ideal.
Proof.
  Split with Mul_Ideal_Pred.
  Exact mul_ideal_theory.
Defined.

Inductive Mul_Ideal_Gen : (x:A) Prop :=
 | mul_ideal_gen : (x,y:A) (I x) -> (J y) -> (Mul_Ideal_Gen (mul x y)).

Lemma mul_ideal_gen_eq : Mul_Ideal==(Ideal_Gen Mul_Ideal_Gen).
Proof.
  Apply ideal_extens.
  Split.
  Unfold ideal_sub; NewInduction 1; Simpl.
  Apply ideal_gen_intro.
  Split; Auto.
  Apply ideal_gen_add; Auto.
  Apply ideal_gen_sub.
  NewDestruct 1; Simpl; Auto.
Qed.

Lemma ideal_sub_mul_cap : (ideal_sub Mul_Ideal (Cap_Ideal I J)).
Proof.
  Rewrite mul_ideal_gen_eq.
  Apply ideal_gen_sub.
  NewInduction 1.
  Split.
  Rewrite mul_commut.
  Auto.
  Auto.
Qed.

End Mul_Ideal.

Require Arith.

(** * Radical of an Ideal. *)

Section Ideal_Rad.

Variable I:Ideal.

(* ** True if [a] belongs to the ideal $(x^n, y^n)$. *)
Inductive RadN [n:nat; x,y:A] : A -> Prop :=
  | RadN_one : n == O -> (RadN n x y one)
  | RadN_zero : (RadN n x y zero)
  | RadN_plus : (a,b:A)
      (RadN n x y a) -> (RadN n x y b) -> (RadN n x y (add a b))
  | RadN_times : (a,b:A)
      (RadN n x y b) -> (RadN n x y (mul a b))
  | RadN_intro : (m,p:nat)
     (plus m p)=n -> (RadN n x y (mul (power x m) (power y p))).

Hints Resolve RadN_one RadN_zero RadN_plus RadN_times RadN_intro.

Remark RadN_x : (x,y:A) (RadN (S O) x y x).
Proof.
  Intros; Pattern 2 x; Replace x with (mul (power x (S O)) (power y O)).
  Auto.
  Replace (power x (1)) with x; Simpl; Auto with arith.
Qed.
Hints Resolve RadN_x.

Remark RadN_y : (x,y:A) (RadN (S O) x y y).
Proof.
  Intros; Pattern 2 y; Replace y with (mul (power x O) (power y (S O))).
  Auto.
  Replace (power y (1)) with y; Simpl; Auto with arith.
Qed.
Hints Resolve RadN_y.

Theorem RadN_mult : (n:nat; x,y:A; a:A)
  (RadN n x y a) -> (RadN (S n) x y (mul (add x y) a)).
Proof.
  NewInduction n; Intros.
  Replace (mul (add x y) a) with (mul a (add x y)); Auto.

  NewInduction H; Simpl; Intros.

  Rewrite H.
  Replace (mul (add x y) one) with (add x y); Auto.

  Replace (mul (add x y) zero) with (!zero A).
  Auto.
  Symmetry; Apply mul_dx_zero.

  Rewrite distrib_sx; Apply RadN_plus; Auto.

  Replace (mul (add x y) (mul a b)) with
    (mul a (mul (add x y) b)); Auto.
  Repeat Rewrite mul_assoc; Auto.

  Rewrite distrib_dx.
  Apply RadN_plus.

  Replace (mul x (mul (power x m) (power y p))) with
    (mul (power x (S m)) (power y p)).
  Apply RadN_intro; Simpl; Auto.
  Simpl; Auto.

  Replace (mul y (mul (power x m) (power y p))) with
    (mul (power x m) (power y (S p))).
  Apply RadN_intro.
  Rewrite <- H.
  Replace (plus m (S p)) with (S (plus m p)); Auto with arith.

  Simpl; Repeat Rewrite mul_assoc; Auto.
Qed.
Hints Resolve RadN_mult.

Remark RadN_power : (n:nat; x,y:A)
  (RadN n x y (power (add x y) n)).
Proof.
  NewInduction n; Intros; Simpl; Auto.
Qed.

Local twice [n:nat] : nat := (plus n n).

Remark lt_plus : (n,m:nat) (lt n m) -> (Ex [u:nat](plus n u)=m).
Proof.
  NewInduction 1.
  Exists (S O); Auto with arith.
  NewInduction n; Simpl; Auto with arith.

  NewInduction IHle.
  Exists (S x).
  Transitivity (S (plus n x)); Auto with arith.
Qed.

Remark twice_property : (n,m,p:nat)
  (plus m p)=(twice n) ->
    (ExT [u:nat](plus n u)=m) \/ (ExT [u:nat](plus n u)=p).
Proof.
  Unfold twice; Intros.
  NewDestruct (lt_eq_lt_dec n m).
  Left; Exists (minus m n).
  NewDestruct s.
  Auto with arith.
  NewDestruct e; Auto with arith.

  Right; Exists (minus p n).
  Apply le_plus_minus_r.
  NewInduction (lt_plus ? ? l).
  NewInduction H0.
  Rewrite plus_assoc_r in H.
  Rewrite (simpl_plus_l ? ? ? H).
  Auto with arith.
Qed.

Remark twice_prep : (n:nat; x,y:A; m,p:nat)
  (plus m p)=(twice n) -> (ExT [a:A] (ExT [b:A]
    (add (mul a (power x n)) (mul b (power y n))) ==
      (mul (power x m) (power y p)))).
Proof.
  Intros.
  LetTac K:=(twice_property ? ? ? H).
  NewInduction K.
  NewInduction H0.
  Exists (mul (power x x0) (power y p)).
  Exists (!zero A).
  Rewrite mul_sx_zero.
  Rewrite zero_dx.
  Rewrite <- H0.
  Rewrite <- power_add; Trivial.
  Replace (mul (power x n) (power x x0)) with
    (mul (power x x0) (power x n)); Auto.
  Repeat Rewrite <- mul_assoc; Auto.

  NewInduction H0.
  Exists (!zero A).
  Exists (mul (power x m) (power y x0)).
  Rewrite mul_sx_zero.
  Rewrite zero_sx.
  Rewrite <- H0.
  Rewrite <- power_add; Trivial.
  Rewrite <- mul_assoc; Auto.
Qed.

Remark RadN_twice : (n:nat; x,y,z:A)
  (RadN (twice n) x y z) ->
    (ExT [a:A](ExT [b:A]
      (add (mul a (power x n)) (mul b (power y n)))==z)).
Proof.
  Unfold twice; NewInduction 1.

  Assert n=O.
  NewInduction n; Trivial; Discriminate.

  Rewrite H0; Simpl.
  Exists (!one A); Exists (!zero A).
  Repeat Rewrite one_dx; Auto.

  Exists (!zero A); Exists (!zero A).
  Do 2 Rewrite mul_sx_zero; Auto.

  NewInduction IHRadN1; NewInduction H0.
  NewInduction IHRadN2; NewInduction H2.
  Exists (add x0 x2); Exists (add x1 x3).
  Rewrite <- H0; Rewrite <- H2.
  Repeat Rewrite distrib_sx Orelse Rewrite distrib_dx.
  Repeat Rewrite add_assoc.
  Apply plug2; Trivial.
  Repeat Rewrite <- add_assoc.
  Auto.

  NewInduction IHRadN; NewInduction H0.
  Exists (mul a x0); Exists (mul a x1).
  Rewrite <- H0.
  Repeat Rewrite distrib_sx; Auto.

  Apply twice_prep.
  Unfold twice; Assumption.
Qed.

Remark twice_pr : (n:nat; x,y:A)
  (ExT [a:A] (ExT [b:A]
    (add (mul a (power x n)) (mul b (power y n))) ==
      (power (add x y) (twice n)))).
Proof.
  Intros.
  Apply RadN_twice.
  Apply RadN_power.
Qed.

Inductive ideal_rad_pred [x:A] : Prop :=
  | ideal_rad_pred_intro : (n:nat)
      (I (power x n)) -> (ideal_rad_pred x).

Hints Resolve ideal_rad_pred_intro.

Remark rad_th : (Ideal_Theory ideal_rad_pred).
Proof.
  Split.

  Replace (!zero A) with (power (!zero A) (S O)).
  Split with (S O); Simpl.
  Repeat Rewrite mul_sx_zero; Auto.
  Simpl; Auto.

  Intros.
  NewDestruct H; NewDestruct H0.
  Exists (twice (plus n n0)).
  NewDestruct (twice_pr (plus n n0) x y).
  NewInduction H1.
  Rewrite <- H1.
  Apply ideal_add.
  Apply ideal_mul.
  Rewrite <- power_add; Auto.
  Rewrite mul_commut; Auto.
  Rewrite <- power_add; Auto.

  Intros; NewInduction H.
  Split with n; Simpl.
  Rewrite power_mul; Auto.
Qed.

Definition ideal_rad : Ideal.
Proof.
  Split with ideal_rad_pred; Exact rad_th.
Defined.

Theorem ideal_rad_one : (ideal_rad one) -> (I one).
Proof.
  NewInduction 1.
  Rewrite power_one in H; Auto.
Qed.

Lemma ideal_rad_power : (x:A; n:nat)
  (ideal_rad (power x n)) -> (ideal_rad x).
Proof.
  NewDestruct 1.
  Split with (mult n n0).
  Rewrite power_power in H; Auto.
Qed.

Lemma ideal_sub_rad : (ideal_sub I ideal_rad).
Proof.
  Unfold ideal_sub; Intros.
  Split with (S O); Simpl.
  Rewrite one_dx; Trivial.
Qed.
End Ideal_Rad.

Lemma ideal_rad_implies : (I,J:Ideal)
  (ideal_sub I J) -> (ideal_sub (ideal_rad I) (ideal_rad J)).
Proof.
  Unfold ideal_sub; Simpl; Intros.
  NewInduction H0.  
  Split with n; Auto.
Qed.
Hints Resolve ideal_rad_implies.

Lemma ideal_rad_sub : (I,J:Ideal)
  (ideal_sub I (ideal_rad J)) -> (ideal_sub (ideal_rad I) (ideal_rad J)).
Proof.
  Unfold ideal_sub; Intros.
  NewDestruct H0.
  Assert HJ := (H ? H0).
  Apply ideal_rad_power with n:=n; Trivial.
Qed.

Lemma ideal_rad_add : (I:Ideal; x,y:A)
  (ExT [n:nat] (I (power x n))) -> (ExT [m:nat] (I (power y m))) ->
  (ExT [p:nat] (I (power (add x y) p))).
Proof.
  NewDestruct 1; NewDestruct 1.

  Assert (ideal_rad I x).
  Split with x0; Auto.

  Assert (ideal_rad I y).
  Split with x1; Auto.  

  Assert (ideal_rad I (add x y)).
  Auto.
  
  NewDestruct H3.
  Split with n; Trivial.
Qed.


End Ideal.

End Ideal.
