(* tag: limits in a category *)

(** * Limits in a category. *)

(** ** Introduction.
  A cone [W] over a (commutative) diagram [D] in a category [A] is
  given by an object [u] of [A] and, for each node [x] of the diagram,
  an arrow, denoted [(cone W x)], from [u] to [x] such that the
  resulting diagram is commutative.

  A cone which is universal is called limit.

  The type of all limit over a diagram [D] is [(Limit D)].
  Limits are unique up to isomorphisms (see: [limit_unique]).
*)

Implicit Arguments On.

Require map_.
Require diagram_.

Module Limit_.

Export Map_.
Export Diagram_.

Section Limit.

Variable A:Cat; D:(CDiagram A).

Section Cone_Theory.

Variable u:A; p:(x:D)(mor u (diagram x)).

Definition Cone_Theory : Prop :=
 (x,y:D; f:(mor (diagram x) (diagram y)))
 (oo f (p x))==(p y).

End Cone_Theory.

Record Cone : Type := {
  cone_vertex :> A;
  cone : (x:D)(mor cone_vertex (diagram x));
  cone_theory : (Cone_Theory cone)
 }.

(** Equality satisfied by the cone morphisms. *)
Lemma cone_eq : (W:Cone; x,y:D; f:(mor (diagram x) (diagram y)))
 (oo f (cone W x))==(cone W y).
Proof.
  Intros.
  Apply cone_theory.
Qed.

(** Exensionality for cones. *)
Lemma Cone_extens : (V,W:Cone)
 (cone_vertex V)==(cone_vertex W) ->
 ((x:D)(eq_mor (cone V x) (cone W x))) ->
  V==W.
Proof.
  NewDestruct V; NewDestruct W; Simpl.
  NewDestruct 1.
  Intros.
  Cut cone0==cone1.
  NewDestruct 1.
  Cut cone_theory0==cone_theory1.  
  NewDestruct 1.
  Reflexivity.
  Apply proof_irrelevance. 
  Apply extensionality with T:=[x:D](mor cone_vertex0 (diagram x)).
  Intros.
  Apply eq_mor_eqT.
  Apply H.
Qed.

(** Extensionality for the projection [Cone]. *)
(* !!! Rewriting of cones do not work very well (see some !!! below).
   This is why we need this lemma.  We should solve this problem and
   erase this lemma. *)
Lemma cone_extens : (V,W:Cone)
  V==W -> (x:D)(eq_mor (cone V x) (cone W x)).
Proof.
  NewDestruct 1; Split.
Qed.

Section Cone_Comp.

Variable W:Cone; u:A; f:(mor u W).

Local g [x:D] : (mor u (diagram x)) := (oo (cone W x) f).

Remark Hg : (Cone_Theory g).
Proof.
  Unfold g.
  Intro; Intros.
  Rewrite oo_assoc.
  Rewrite cone_eq.
  Reflexivity.
Qed.

Definition cone_comp : Cone.
Proof.
  Split with cone:=g; Exact Hg.
Defined.

End Cone_Comp.

Record Limit : Type := {
  limit_cone :> Cone;
  limit : (u:A)(inverse [f:(mor u limit_cone)](cone_comp f))
 }.

Definition limit_mor [U:Limit; W:Cone] : (mor W U) :=
 (limit U W W).

Lemma limit_eq : (U:Limit; W:Cone; x:D)
 (oo (cone U x) (limit_mor U W))==(cone W x).
Proof.
  Intros.
  Change (cone (cone_comp (limit_mor U W)) x)==(cone W x).
  Unfold limit_mor.
  (* !!! too difficult. *)
  Exact (eq_mor_eqT (cone_extens (inverse_dx_eq (limit U W) W) x)).
Qed.

Hints Resolve cone_eq limit_eq.
Hint Rewrite [cone_eq limit_eq] in limit.

Section Limit_Mor_Unique.

Variable U:Limit; W:Cone.

Lemma limit_mor_char :
 (f:(mor W U); Hf:(x:D)(cone W x)==(oo (cone U x) f))
  (limit_mor U W)==f.
Proof.
  Intros.  
  Rewrite <- (inverse_sx_eq (limit U W) f).
  Replace (cone_comp f) with W.
  Reflexivity.
  Apply Cone_extens; Simpl; Intros; Try Reflexivity.
  Rewrite <- Hf.
  Split.
Qed.

Variable f,g:(mor W U).
Hypothesis Hf:(x:D)(cone W x)==(oo (cone U x) f).
Hypothesis Hg:(x:D)(cone W x)==(oo (cone U x) g).

Lemma limit_mor_unique : f==g.
Proof.
  Transitivity (limit_mor U W).
  Rewrite limit_mor_char with f; Trivial.
  Rewrite limit_mor_char with g; Trivial.
Qed.

End Limit_Mor_Unique.

Remark limit_mor_ii : (U,V:Limit)
 (oo (limit_mor V U) (limit_mor U V))==(ii V).
Proof.
  Intros.
  Apply limit_mor_unique; Intros.
  Rewrite oo_assoc.
  Do 2 Rewrite limit_eq.
  Reflexivity.
  Rewrite ii_dx.
  Reflexivity. 
Qed.

Definition limit_unique : (U,V:Limit)(Inverse_Mor (limit_mor U V)).
Proof.
  Intros.
  Split with (limit_mor V U).
  Abstract Split; Rewrite limit_mor_ii; Reflexivity.
Defined.

End Limit.

Hints Resolve cone_eq limit_eq.
Hint Rewrite [cone_eq limit_eq] in limit.

End Limit_.
