(* tag: Lists *)

(** * Lists *)

Require Export Arith.

Implicit Arguments On.

Module List_.

Section List.

Theorem plug : (A:Type; f:A->A; x,y:A)
  x==y -> (f x)==(f y).
Proof.
NewDestruct 1; Reflexivity.
Qed.
Hints Resolve plug.

Record Prod [A,B:Type] : Type := pair {
  pair1 : A;
  pair2 : B
}.

Section List.
  Variable A:Type.

  Inductive List : Type :=
  | nil : List
  | cons : A->List->List.

  Definition is_nil [l:List] : Prop :=
    Cases l of nil => True | xx => False end.

  Definition uncons [l:List] : List :=
    Cases l of nil => nil | (cons xx t) => t end.

  Lemma cons_plug : (x,y:A; a,b:List)
    x==y -> a==b -> (cons x a)==(cons y b).
  Proof.
  Induction 1; Induction 1; Reflexivity.
  Qed.
  Hints Resolve cons_plug.

  Fixpoint append [a:List] : List->List :=
    [b:List] Cases a of
    | nil => b
    | (cons h t) => (cons h (append t b))
    end.

  Fixpoint revappend [a:List] : List->List :=
    [b:List] Cases a of
    | nil => b
    | (cons h t) => (revappend t (cons h b))
    end.

  Definition reverse [a:List] : List := (revappend a nil).
  Hints Unfold reverse.

  Theorem append_assoc : (a,b,c:List)
    (append a (append b c))==(append (append a b) c).
  Proof.
  NewInduction a; Simpl; Auto.
  Qed.

  Lemma append_nil : (a:List) (append a nil)==a.
  Proof.
  NewInduction a; Simpl; Auto.
  Qed.
  Hints Resolve append_nil.

  Theorem revappend_assoc : (a,b,c:List)
    (append (revappend a b) c)==(revappend a (append b c)).
  Proof.
  NewInduction a; Simpl; Auto.
  Intros; Rewrite IHa; Trivial.
  Qed.
  Hints Resolve revappend_assoc.

  Lemma revappend_prop : (a,b:List)
    (revappend a b)==(append (reverse a) b).
  Proof.
  Unfold reverse; NewInduction a; Intros; Simpl; Trivial.
  Rewrite revappend_assoc; Auto.
  Qed.
  Hints Resolve revappend_prop.

  Fixpoint enqueue [x:A; a:List] : List :=
    Cases a of
    | nil => (cons x nil)
    | (cons h t) => (cons h (enqueue x t))
    end.

  Lemma enqueue_prop : (x:A; a:List)
    (enqueue x a)==(append a (cons x nil)).
  Proof.
  NewInduction a; Simpl; Auto.
  Qed.
  Hints Resolve enqueue_prop.

  Lemma enqueue_append_prop : (x:A; a,b:List)
    (enqueue x (append a b))==(append a (enqueue x b)).
  Proof.
  NewInduction a; Intros; Simpl; Trivial.
  Rewrite IHa; Auto.
  Qed.

  Lemma enqueue_revappend_prop : (x:A; a,b:List)
    (enqueue x (revappend a b))==(revappend a (enqueue x b)).
  Proof.
  NewInduction a; Intros; Simpl; Trivial.
  Rewrite IHa; Auto.
  Qed.

  Lemma enqueue_reverse_prop : (x:A; a:List)
    (enqueue x (reverse a))==(reverse (cons x a)).
  Proof.
  Unfold reverse; Intros.
  Rewrite enqueue_revappend_prop; Trivial.
  Qed.

  Lemma list_disc : (a:List) a==nil -> (is_nil a).
  Proof.
    Intros; Rewrite H; Split.
  Qed.

  Lemma neq_cons_nil : (x:A; a:List) ~(cons x a)==nil.
  Proof.
  Red; Intros.
  Apply list_disc with a:=(cons x a).
  Assumption.
  Qed.

  Theorem is_nil_dec : (a:List) a==nil\/~a==nil.
  Proof.
  NewDestruct a.
  Left; Split.
  Right; Red; Intros.
  Apply neq_cons_nil with x:=a a:=l.
  Assumption.
  Qed.
End List.

Hints Resolve cons_plug append_assoc append_nil : list.

Section Length.
  Variable A:Type.

  (* Local *) Fixpoint length_aux [a:(List A)] : nat->nat :=
    [n:nat]Cases a of
    | nil => n
    | (cons xx xs) => (length_aux xs (S n))
    end.

  Definition length [a:(List A)] : nat := (length_aux a O).

  Fixpoint length_no_tail [a:(List A)] : nat :=
    Cases a of
    | nil => O
    | (cons xx xs) => (S (length_no_tail xs))
    end.

  Remark length_aux_rmk : (a:(List A); n:nat)
    (length_aux a (S n))==(S (length_aux a n)).
  Proof.
  NewInduction a; Simpl; Trivial.
  Qed.

  Lemma length_compute : (a:(List A))
    (length a)==(length_no_tail a).
  Proof.
  Unfold length; NewInduction a; Simpl; Trivial.
  Rewrite <- IHa.
  Rewrite length_aux_rmk.
  Reflexivity.
  Qed.
End Length.

Fixpoint nth [A:Type; o:A; n:nat; l:(List A)] : A :=
  Cases n l of
  | n nil => o
  | O (cons x xx) => x
  | (S m) (cons x l1) => (nth o m l1)
  end.

Fixpoint map [A,B:Type; f:A->B; l:(List A)] : (List B) :=
  Cases l of
  | nil => (nil B)
  | (cons h t) => (cons (f h) (map f t))
  end.

Lemma map_comb : (A,B,C:Type; f:A->B; g:B->C; xs:(List A))
  (map g (map f xs))==(map [x:A](g (f x)) xs).
Proof.
NewInduction xs; Simpl; Auto with list.
Qed.

Lemma nth_map : (A,B:Type; f:A->B; o:A; l:(List A); n:nat)
  (nth (f o) n (map f l))==(f (nth o n l)).
Proof.
NewInduction l; NewDestruct n; Simpl; Trivial.
Qed.

Fixpoint revmap_aux [A,B:Type; f:A->B; l:(List A)] : (List B)->(List B) :=
  [acc:(List B)] Cases l of
  | nil => acc
  | (cons h t) => (revmap_aux f t (cons (f h) acc))
  end.

Definition revmap [A,B:Type; f:A->B; l:(List A)] : (List B) :=
  (revmap_aux f l (nil B)).

Lemma map_append_distrib : (A,B:Type; f:A->B; a,b:(List A))
  (map f (append a b))==(append (map f a) (map f b)).
Proof.
NewInduction a; Simpl; Auto.
Qed.

Lemma map_length : (A,B:Type; f:A->B; a:(List A))
  (length (map f a))==(length a).
Proof.
Intros.
Repeat Rewrite length_compute.
NewInduction a; Simpl; Trivial.
Rewrite IHa; Reflexivity.
Qed.

Section Iter.
  Variable A,B:Type.
  Variable comb:A->B->B.

  Fixpoint fold_aux [a:(List A)] : B->B :=
    [acc:B] Cases a of
    | nil => acc
    | (cons x xs) => (fold_aux xs (comb x acc))
    end.

  Definition fold [init:B; a:(List A)] : B := (fold_aux a init).
End Iter.

Section Tabulate.
  Variable A:Type.
  Variable init:nat->A.

   Fixpoint tabulate_aux [n:nat] : (List A) -> (List A) :=
    [acc:(List A)]Cases n of
    | O => acc
    | (S n) => (tabulate_aux n (cons (init (S n)) acc))
    end.

  Definition tabulate [n:nat] : (List A) := (tabulate_aux n (nil A)).

  Remark rmk1 : (n:nat; a:(List A))
    (length_no_tail (tabulate_aux n a))=(plus n (length_no_tail a)).
  Proof.
  NewInduction n; Simpl; Trivial.
  NewInduction a; Simpl; Rewrite IHn; Simpl; Auto with arith.
  Qed.

  Lemma tabulate_length : (n:nat)
    (length (tabulate n))=n.
  Proof.
  Intros; Rewrite length_compute; Unfold tabulate.
  NewInduction n; Simpl; Trivial.
  Rewrite rmk1; Simpl.
  Clear IHn; NewInduction n; Simpl; Auto with arith.
  Qed.
End Tabulate.

Definition not_nil [A:Type; xs:(List A)] : Prop :=
  Cases xs of nil => False | xx => True end.

Lemma cons_not_nil : (A:Type; x:A; xs:(List A)) (not_nil (cons x xs)).
Proof.
Intros; Simpl; Trivial.
Qed.

Definition car [A:Type; default:A; l:(List A)] :=
  Cases l of nil => default | (cons x xx) => x end.

Definition cdr [A:Type; l:(List A)] :=
  Cases l of nil => (nil A) | (cons xx xs) => xs end.

Lemma cdr_not_nil : (A:Type; l:(List A))
  (not_nil (cdr l)) -> (not_nil l).
Proof.
NewInduction l.
Tauto.
Simpl.
NewDestruct l; Tauto.
Qed.

Definition not_nil_car [A:Type; xs:(List A); nn:(not_nil xs)] : A.
Proof.
Intros.
NewDestruct xs.
Elim nn.
Exact a.
Defined.

Remark not_nil_car_ok : (A:Type; x:A; xs:(List A); nn:(not_nil (cons x xs)))
  (not_nil_car nn)==x.
Proof.
Intros; Simpl; Trivial.
Qed.

(** [(pairs xs ys)] build the list of ordered pairs of elements of [xs], [ys]. *)
Fixpoint pairs [A,B:Type; l:(List A)] : (List B)->(List (Prod A B)) :=
  [ys:(List B)]
  Cases l of
  | nil => (nil (Prod A B))
  | (cons x xs) =>
      (append (map (!pair A B x) ys) (!pairs A B xs ys))
  end.

(** [(for_all P xs)] holds if [(P x)] holds for all elements [x] of [xs]. *)
Inductive for_all [A:Type; P:A->Prop] : (List A)->Prop :=
| for_all_intro_nil : (!for_all A P (nil A))
| for_all_intro_cons : (x:A; xs:(List A))
   (P x)->(!for_all A P xs)->(!for_all A P (cons x xs)).
Hints Resolve for_all_intro_nil for_all_intro_cons : list.

Lemma for_all_head : (A:Type; P:A->Prop; x:A; xs:(List A))
  (for_all P (cons x xs)) -> (P x).
Proof.
Intros.
Inversion H; Trivial.
Qed.

Lemma for_all_tail : (A:Type; P:A->Prop; x:A; xs:(List A))
  (for_all P (cons x xs)) -> (for_all P xs).
Proof.
Intros.
Inversion H; Trivial.
Qed.

Lemma for_all_trivial : (A:Type; P:A->Prop; xs:(List A))
  ((a:A)(P a)) -> (for_all P xs).
Proof.
NewInduction xs; Auto with list.
Qed.

Lemma for_all_implies : (A:Type; P,Q:A->Prop; xs:(List A))
  ((a:A)(P a)->(Q a)) -> (for_all P xs) -> (for_all Q xs).
Proof.
NewInduction xs; Intros; Auto with list.
Apply for_all_intro_cons.
Apply H.
Apply (for_all_head H0).
Apply IHxs; Trivial.
Apply (for_all_tail H0).
Qed.

(** Map over two lists. *)
Section Map2.
  Variable A,B,C:Type.
  Variable f:A->B->C.
  Variable oA:A; oB:B.

  Fixpoint map2 [xs:(List A); ys:(List B)] : (List C) :=
    Cases xs ys of
    | (cons x xs1) (cons y ys1) => (cons (f x y) (map2 xs1 ys1))
    | xx nil => (map [x:A](f x oB) xs)
    | nil xx => (map [y:B](f oA y) ys)
    end.
End Map2.

Section Same_Length.
  Variable A,B:Type.

  Fixpoint same_length [xs:(List A); ys:(List B)] : Prop :=
    Cases xs ys of
    | nil nil => True
    | nil xx => False
    | xx nil => False
    | (cons xx xs1) (cons xx1 ys1) => (same_length xs1 ys1)
    end.

  Lemma same_length_not_nil : (xs:(List A); ys:(List B))
    (same_length xs ys)->(not_nil xs)->(not_nil ys).
  Proof.
  NewDestruct xs; NewDestruct ys; Intros; Simpl; Trivial.
  Qed.

  Lemma same_length_eq : (xs:(List A); ys:(List B))
    (same_length xs ys)->(length xs)==(length ys).
  Proof.
  NewInduction xs; NewDestruct ys; Simpl; Try Tauto.
  Do 2 Rewrite length_compute; Simpl.
  Do 2 Rewrite <- length_compute.
  Auto.
  Qed.

  Lemma map_same_length : (f:A->B; xs:(List A))
    (same_length xs (map f xs)).
  Proof.
  NewInduction xs; Simpl; Trivial.
  Qed.
End Same_Length.

End List.

End List_.
