(* tag: posets *)

(** * Posets *)

Implicit Arguments On.

Require misc.

Module Poset.

Import Misc.

Syntactic Definition AX_Reflexivity :=
 [A:Type; P:A->A->Prop] (x:A) (P x x) | 1.

Syntactic Definition AX_Symmetry :=
 [A:Type; P:A->A->Prop]
 (x,y:A) (P x y) -> (P y x) | 1.

Syntactic Definition AX_Anti_Symmetry :=
 [A:Type; P:A->A->Prop]
 (x,y:A) (P x y) -> (P y x) -> x==y| 1.

Syntactic Definition AX_Transitivity :=
 [A:Type; P:A->A->Prop]
 (x,y,z:A) (P x y) -> (P y z) -> (P x z) | 1.

Definition Ax_Reflexivity :=
  Eval Simpl in [A:Type; P:A->A->Prop](AX_Reflexivity P).

Hints Unfold Ax_Reflexivity.

Definition Ax_Symmetry :=
  Eval Simpl in [A:Type; P:A->A->Prop](AX_Symmetry P).

Hints Unfold Ax_Symmetry.

Definition Ax_Anti_Symmetry :=
  Eval Simpl in [A:Type; P:A->A->Prop](AX_Anti_Symmetry P).

Hints Unfold Ax_Anti_Symmetry.

Definition Ax_Transitivity :=
  Eval Simpl in [A:Type; P:A->A->Prop](AX_Transitivity P).

Hints Unfold Ax_Transitivity.

Definition Ax_Poset :=
  [A:Type; P:A->A->Prop]
  (Ax_Reflexivity P) /\ (Ax_Anti_Symmetry P) /\ (Ax_Transitivity P).

Hints Unfold Ax_Poset.

Record Poset : Type := {
  poset_carrier :> Type;
  leq : poset_carrier->poset_carrier->Prop;
  ax_poset : (Ax_Poset leq)
 }.

Section Poset_Facts.

Variable A:Poset.
Local HA := (ax_poset A). 

Local leq_reflexivity_goal :=
  Eval Simpl in (AX_Reflexivity (!leq A)).
Lemma leq_reflexivity : leq_reflexivity_goal.
Proof.
  Unfold leq_reflexivity_goal.
  NewDestruct HA; Auto.
Qed.

Local leq_anti_symmetry_goal :=
  Eval Simpl in (AX_Anti_Symmetry (!leq A)).
Lemma leq_anti_symmetry : leq_anti_symmetry_goal.
Proof.
  Unfold leq_anti_symmetry_goal.
  NewDestruct HA.
  NewDestruct H0.  
  Auto.
Qed.

Local leq_transitivity_goal :=
  Eval Simpl in (AX_Transitivity (!leq A)).
Lemma leq_transitivity : leq_transitivity_goal.
Proof.
  Unfold leq_transitivity_goal.
  NewDestruct HA; NewDestruct H0; Auto.
Qed.

End Poset_Facts.

Hints Resolve leq_reflexivity leq_anti_symmetry leq_transitivity.

Section Poset_Morph.

Variable A,B:Poset.

Syntactic Definition AX_Non_Decreasing :=
  [f:A->B](x,y:A)(leq x y) -> (leq (f x) (f y)).

Definition Ax_Non_Decreasing : (f:A->B) Prop :=
  Eval Simpl in [f:A->B](AX_Non_Decreasing f).

Record Poset_Morph : Type := {
  poset_morph_carrier :> A ->B;
  ax_non_decreasing :
    (Ax_Non_Decreasing poset_morph_carrier)
 }.

Local non_decreasing_goal :=
  Eval Simpl in (f:(Poset_Morph))(AX_Non_Decreasing f).
Lemma non_decreasing : non_decreasing_goal.
Proof.
  Unfold non_decreasing_goal.
  NewDestruct f; Assumption.
Qed.

End Poset_Morph.

Hints Unfold Ax_Non_Decreasing.
Hints Resolve non_decreasing.

Section Sub_Poset.

Variable A:Poset; P:A->Prop.

Local B := (Set_Of P).

Local leqB [x,y:B] : Prop :=
  (leq (forget x) (forget y)).

Remark leqB_anti_sym : (Ax_Anti_Symmetry leqB).
Proof.
  Unfold Ax_Anti_Symmetry.
  NewDestruct x; NewDestruct y; Simpl.
  Intros.
  Cut x0==x.
  NewDestruct 1.
  Cut p0==p.
  NewDestruct 1.
  Reflexivity.
  Apply proof_irrelevance.
  Auto.
Qed.

Definition Sub_Poset : Poset.
Proof.
  Split with leq:=[x,y:B](leq (forget x) (forget y)).
  Hints Resolve leqB_anti_sym.
  Abstract Split; Try Split; EAuto.
Defined.

Definition sub_poset_morph : (Poset_Morph Sub_Poset A).
Proof.
  Split with [x:Sub_Poset](forget x).
  Auto.
Qed.

End Sub_Poset.

End Poset.
