(* tag: preordered sets *)

(** * Preordered Sets *)

Implicit Arguments On.

Require misc_.

Module Preordered_.

Import Misc_.

Syntactic Definition AX_Reflexivity :=
 [A:Type; P:A->A->Prop] (x:A) (P x x) | 1.

Syntactic Definition AX_Symmetry :=
 [A:Type; P:A->A->Prop]
 (x,y:A) (P x y) -> (P y x) | 1.

Syntactic Definition AX_Transitivity :=
 [A:Type; P:A->A->Prop]
 (x,y,z:A) (P x y) -> (P y z) -> (P x z) | 1.

Definition Ax_Reflexivity :=
  Eval Simpl in [A:Type; P:A->A->Prop](AX_Reflexivity P).

Hints Unfold Ax_Reflexivity.

Definition Ax_Symmetry :=
  Eval Simpl in [A:Type; P:A->A->Prop](AX_Symmetry P).

Hints Unfold Ax_Reflexivity.

Definition Ax_Transitivity :=
  Eval Simpl in [A:Type; P:A->A->Prop](AX_Transitivity P).

Hints Unfold Ax_Transitivity.

Definition Ax_Preordered :=
  [A:Type; P:A->A->Prop]
  (Ax_Reflexivity P) /\ (Ax_Transitivity P).

Hints Unfold Ax_Preordered.

Record Preordered : Type := {
  preordered_carrier :> Type;
  leq : preordered_carrier->preordered_carrier->Prop;
  ax_preordered : (Ax_Preordered leq)
 }.

Section Preordered_Facts.

Variable A:Preordered.
Local HA := (ax_preordered A). 

Local leq_reflexivity_goal :=
  Eval Simpl in (AX_Reflexivity (!leq A)).
Lemma leq_reflexivity : leq_reflexivity_goal.
Proof.
  Unfold leq_reflexivity_goal.
  NewDestruct HA; Auto.
Qed.

Local leq_transitivity_goal :=
  Eval Simpl in (AX_Transitivity (!leq A)).
Lemma leq_transitivity : leq_transitivity_goal.
Proof.
  Unfold leq_transitivity_goal.
  NewDestruct HA; Auto.
Qed.

End Preordered_Facts.

Hints Resolve leq_reflexivity leq_transitivity.

Section Preordered_Morph.

Variable A,B:Preordered.

Syntactic Definition AX_Non_Decreasing :=
  [f:A->B](x,y:A)(leq x y) -> (leq (f x) (f y)).

Definition Ax_Non_Decreasing : (f:A->B) Prop :=
  Eval Simpl in [f:A->B](AX_Non_Decreasing f).

Record Preordered_Morph : Type := {
  preordered_morph_carrier :> A ->B;
  ax_non_decreasing :
    (Ax_Non_Decreasing preordered_morph_carrier)
 }.

Local non_decreasing_goal :=
  Eval Simpl in (f:(Preordered_Morph))(AX_Non_Decreasing f).
Lemma non_decreasing : non_decreasing_goal.
Proof.
  Unfold non_decreasing_goal.
  NewDestruct f; Assumption.
Qed.

End Preordered_Morph.

Hints Unfold Ax_Non_Decreasing.
Hints Resolve non_decreasing.

Section Sub_Preordered.

Variable A:Preordered; P:A->Prop.

Local B := (Set_Of P).

Definition Sub_Preordered : Preordered.
Proof.
  Split with leq:=[x,y:B](leq (forget x) (forget y)).
  Abstract Split; EAuto.
Defined.

Definition sub_preordered_morph : (Preordered_Morph Sub_Preordered A).
Proof.
  Split with [x:Sub_Preordered](forget x).
  Auto.
Qed.

End Sub_Preordered.

End Preordered_.
