(* tag: localization of rings *)

(** * Localization of rings. *)

(* !!! TODO: Definition -> Local *)

Implicit Arguments On.

Require Export cring.
Require quot.
Require Ring.

Module Ring_Localization.

Definition Coq_Ring_Theory := Ring_Theory.

Export CRing.
Import Quot.

Lemma coq_ring_theory : (R:CRing)
 (Coq_Ring_Theory (!add R) (!mul R) one zero (!opp R) [x,y:R]false).
Proof.
  Split; Intros; Auto.
  Simpl in H; Tauto.
Qed.

Section Reciproc.

Variable R:CRing.

Record Reciproc [x:R] : Type := {
  reciproc_elt :> R;
  reciproc_prf : (mul reciproc_elt x)==one
 }.

Hints Resolve reciproc_prf.

Lemma cancel_mul_sx : (x,y,z:R; u:(Reciproc x))
 (mul x y)==(mul x z) -> y==z.
Proof.
  Intros.
  Rewrite <- one_sx with x:=y.
  Rewrite <- one_sx with x:=z.
  Rewrite <- reciproc_prf with x u.
  Do 2 Rewrite <- mul_assoc.
  NewDestruct H.
  Reflexivity.
Qed.

Definition reciproc_reciproc [x:R; u:(Reciproc x)] :
 (Reciproc u).
Proof.
  Intros; Split with x.
  Abstract Rewrite mul_commut; Auto.
Defined.

End Reciproc.

Hints Resolve reciproc_prf.

Section Invertible.

Variable R:CRing.

Definition Invertible [x:R] : Prop :=
 (EXT y:R | (mul x y)==one).

End Invertible.

Section Ring_Localization.

Variable R:CRing.

Definition R_type : Type := R.
Definition Coq_RT := (coq_ring_theory R).
Definition Radd : R_type->R_type->R_type := (!add R).
Definition Rzero : R_type := zero.
Definition Ropp : R_type->R_type := (!opp R).
Definition Rmul : R_type->R_type->R_type := (!mul R).
Definition Rone : R_type := one.
Add Abstract Ring R_type Radd Rmul Rone Rzero Ropp
 [x,y:R]false Coq_RT.

Section Mul_System_Theory.

Variable P : R->Prop.

Record Mul_System_Theory : Prop := {
  ax_mul_system_one : (P one);
  ax_mul_system_mul : (x,y:R) (P x) -> (P y) -> (P (mul x y))
 }.

End Mul_System_Theory.

Record Mul_System : Type := {
  mul_system_pred :> R -> Prop;
  mul_system_theory : (Mul_System_Theory mul_system_pred)
 }.

Section Mul_System_Facts.

Variable P : Mul_System.
Local HP := (mul_system_theory P).

Lemma mul_system_one : (P one).
Proof.
  NewDestruct HP; Trivial.
Qed.

Lemma mul_system_mul : (x,y:R) (P x) -> (P y) -> (P (mul x y)).
Proof.
  NewDestruct HP; Trivial.
Qed.

End Mul_System_Facts.

Hints Resolve mul_system_one mul_system_mul.

Variable P : Mul_System.

Record Frac : Type := frac {
  num : R;
  den : R;
  den_prf : (P den)
 }.

Implicits frac [].

Hints Resolve den_prf.

Remark den_mul_prf : (x,y:Frac)
 (P (Rmul (den x) (den y))).
Proof.
  Unfold Rmul; Auto.
Qed.

Hints Resolve den_mul_prf.

Definition Qrel [x,y:Frac] : Prop :=
  (EXT s |
    (P s) /\
    (Rmul s (Rmul (den y) (num x)))==(Rmul s (Rmul (den x) (num y)))).

Remark qrel_simple : (x,y:Frac)
 (Rmul (den y) (num x))==(Rmul (den x) (num y)) -> (Qrel x y).
Proof.
  Intros.
  Exists Rone.
  NewDestruct H.
  Trivial.
  Split.
  Trivial.
  Ring.
Qed.

Remark qrel_trivial : (x,y:Frac)
 (num x)==(num y) -> (den x)==(den y) -> (Qrel x y).
Proof.
  Intros; Apply qrel_simple.
  NewDestruct H; NewDestruct H0.
  Reflexivity.
Qed.

Remark HQ : (Equivalence_Theory Qrel).
Proof.
  Split; Simpl; Intros.
  Apply qrel_trivial; Reflexivity.
  NewDestruct H.
  Exists x0.
  Split; Intuition.
  NewDestruct H; NewDestruct H0.
  Exists (Rmul x0 (Rmul x1 (den y))).
  Split.
  Intuition.
  Transitivity (Rmul x1 (Rmul (den z)
     (Rmul x0 (Rmul (den y) (num x))))).
  Ring.
  NewDestruct H.
  Rewrite H1.
  Transitivity (Rmul x0 (Rmul (den x)
     (Rmul x1 (Rmul (den z) (num y))))).
  Ring.
  NewDestruct H0.
  Rewrite H2.
  Ring.
Qed.

Definition Q : (Equivalence Frac).
Proof.
  Split with Qrel; Exact HQ.
Defined.

Local i [x:R_type] : Frac.
Proof.
  Intros; Apply (frac x Rone); Auto.
Defined.

Definition Fmul [x,y:Frac] : Frac.
Proof.
  Intros.
  Apply (frac (Rmul (num x) (num y)) (Rmul (den x) (den y))).
  Abstract Auto.
Defined.

Definition Fadd [x,y:Frac] : Frac.
Proof.
  Intros.
  Apply (frac (Radd (Rmul (den y) (num x)) (Rmul (den x) (num y)))
   (Rmul (den x) (den y))).
  Abstract Auto.
Defined.

Definition Fopp [x:Frac] : Frac.
Proof.
  Intros.
  Apply (frac (Ropp (num x)) (den x)).
  Abstract Auto.
Defined.

Definition S_type := (quot Q).

Remark Fadd_compat :
 (Compatible2 Q Q [x,y:Frac](class Q (Fadd x y))).
Proof.
  Unfold Compatible2; Intros.
  Apply class_eq.
  NewDestruct H.
  NewDestruct H0.
  Exists (Rmul x x0).
  Split.
  Intuition.
  Simpl; Ring.

  Replace 
   (Rmul (den x2)
     (Rmul (den y2)
       (Rmul x (Rmul (num y1) (Rmul (den x1) x0)))))
  with
   (Rmul x
     (Rmul (den x2)
       (Rmul (den x1)
	 (Rmul x0 (Rmul (den y2) (num y1)))))).
  NewDestruct H0.
  Rewrite H1.
  2:Ring.
  Replace
   (Rmul (den y2)
     (Rmul x
       (Rmul (den y1)
	 (Rmul (num x2) (Rmul (den x1) x0)))))
  with
   (Rmul x0
     (Rmul (den y2)
       (Rmul (den y1)
	 (Rmul x (Rmul (den x1) (num x2)))))).
  NewDestruct H.
  NewDestruct H2.
  2:Ring.
  Ring.
Qed.

Definition Sadd [x,y:S_type] : S_type.
Proof.
  Apply (quot_law2 Q Fadd).
  Apply Fadd_compat.
Defined.

Remark Fmul_compat :
 (Compatible2 Q Q [x,y:Frac](class Q (Fmul x y))).
Proof.
  Intro; Intros.
  Apply class_eq.
  NewDestruct H; NewDestruct H0.
  Exists (Rmul x x0).
  Simpl.
  Split.
  Intuition.
  Transitivity
    (Rmul
      (Rmul x (Rmul (den x2) (num x1)))
      (Rmul x0 (Rmul (den y2) (num y1)))).
  Ring.
  NewDestruct H.
  Rewrite H1.
  NewDestruct H0.
  Rewrite H2.
  Ring.
Qed.

Definition Smul [x,y:S_type] : S_type.
Proof.
  Apply (quot_law2 Q Fmul).
  Apply Fmul_compat.
Defined.

Remark Fopp_compat :
 (Compatible Q [x:Frac](class Q (Fopp x))).
Proof.
  Intro; Intros.
  Apply class_eq.
  NewDestruct H.
  Exists x0.
  Split; Simpl.
  Intuition.
  Transitivity (Ropp (Rmul x0 (Rmul (den y) (num x)))).
  Ring.
  NewDestruct H.
  Rewrite H0.
  Ring.
Qed.

Definition Sopp [x:S_type] : S_type.
Proof.
  Apply (quot_law Q Fopp).
  Apply Fopp_compat.
Defined.

Definition Szero : S_type := (class Q (i Rzero)).

Definition Sone : S_type := (class Q (i Rone)).

Definition S_data : Ring_Data :=
 (Build_Ring_Data Sadd Szero Sopp Smul Sone).

Remark HS : (CRing_Theory Sadd Szero Sopp Smul Sone).
Proof.
  Unfold Sadd Szero Sopp Smul Sone.
  Unfold quot_law quot_law2.

  (* Now keep your breath... *)
  Split; Simpl; Intros;
  NewDestruct (class_surj x); NewDestruct H;
  Try (NewDestruct (class_surj y); NewDestruct H);
  Try (NewDestruct (class_surj z); NewDestruct H);
  Repeat (Rewrite fact2_eq Orelse Rewrite fact_eq);
  Apply class_eq with Q:=Q;
  Apply qrel_simple; Simpl; Ring.
Qed.

Definition Localized_Ring : CRing.
Proof.
  Split with S_data; Exact HS.
Defined.

(** Canonical map of a ring to its localization. *)
Definition pp [x:R] : Localized_Ring := (class Q (i x)).

Remark pp_theory : (Morph_Theory pp).
Proof.
  Unfold pp.
  Split; Simpl;
    Unfold Sadd Szero Sopp Smul Sone quot_law quot_law2; Intros;
    Repeat (Rewrite fact2_eq Orelse Rewrite fact_eq);
    Apply class_eq with Q:=Q;
    Apply qrel_trivial; Simpl;
    Unfold Radd Rzero Ropp Rmul Rone;
    Repeat (Rewrite one_sx Orelse Rewrite one_dx); Trivial.
Qed.

Definition loc : (Morph R Localized_Ring).
Proof.
  Split with pp; Exact pp_theory.
Qed.

(** ** Universal property of localized rings. *)
Section Factorization.

Variable T:CRing.

Definition T_type : Type := T.
Definition Coq_TT := (coq_ring_theory T).
Definition Tadd : T_type->T_type->T_type := (!add T).
Definition Tzero : T_type := zero.
Definition Topp : T_type->T_type := (!opp T).
Definition Tmul : T_type->T_type->T_type := (!mul T).
Definition Tone : T_type := one.
Add Abstract Ring T_type Tadd Tmul Tone Tzero Topp
 [x,y:T]false Coq_TT.

Variable f:(Morph R T).
Variable g:(x:R) (P x)->(Reciproc (f x)).

Definition ff [x:Frac] : T_type :=
 (Tmul (reciproc_elt (g (den_prf x)))(f (num x))).

Remark ff_compat : (Compatible Q ff).
Proof.
  Intro; Intros.
  NewDestruct H.
  NewDestruct H.
  Unfold ff Tmul T_type.
  Apply cancel_mul_sx with (f (den x)).
  Exact (g (den_prf x)).
  Do 2 Rewrite mul_assoc.
  Rewrite (mul_commut (f (den x)) (g (den_prf x))).
  Rewrite reciproc_prf.
  Rewrite one_sx.
  Rewrite (mul_commut (f (den x)) (g (den_prf y))).
  Apply cancel_mul_sx with (f (den y)).
  Exact (g (den_prf y)).
  Repeat Rewrite mul_assoc.
  Rewrite (mul_commut (f (den y)) (g (den_prf y))).
  Rewrite reciproc_prf.
  Rewrite one_sx.
  Apply cancel_mul_sx with (f x0).
  Exact (g H).
  Repeat Rewrite <- (morph_mul f).
  Unfold Rmul in H0.
  NewDestruct H0.
  Reflexivity.
Qed.

Definition h [x:Localized_Ring] : T.
Proof.
  Apply fact with Q:=Q f:=ff.
  Exact ff_compat.
Defined.

End Factorization.

End Ring_Localization.

End Ring_Localization.
