(* tag: Sheafs over a site *)

Require Export presheaf_.
Require Export topology_.
Require Export small_set_.
Require misc_.
Require map_.

Implicit Arguments On.

(** * Sheaves over a site. *)

Module Sheaf_.

Export Presheaf_.
Export Topology_.
Export Small_Set_.
Import Misc_.
Import Map_.

Section Sheaf.

Variable A:Site.

(** A sheaf is a presheaf for which any matching family has an unique
    amalgamation. *)

Section Matching_Family.

Variable P:(Presheaf A Small_Set).
Variable u:A; s:(Sieve u).

Definition Matching_Family_Data :=
 (x:A; f:(mor x u); Hf:(sieve s f)) (P x).

Definition Matching_Family_Prop [p:Matching_Family_Data] : Prop :=
 (x:A; f:(mor x u); Hf:(sieve s f))
 (y:A; g:(mor y x); Hfg:(sieve s (oo f g)))
 (presh P g (p x f Hf)) == (p y (oo f g) Hfg).

Record Matching_Family : Type := {
  matching_family_data :> (x:A; f:(mor x u); Hf:(sieve s f)) (P x);
  ax_matching_family_prop : (Matching_Family_Prop matching_family_data)
 }.

Lemma matching_family_prop : (p:Matching_Family)
 (x:A; f:(mor x u); Hf:(sieve s f))
 (y:A; g:(mor y x); Hfg:(sieve s (oo f g)))
 (presh P g (p x f Hf)) == (p y (oo f g) Hfg).
Proof.
  NewDestruct p; Auto.
Qed.

Hints Resolve matching_family_prop.

Lemma matching_family_congr : (p:Matching_Family; x:A)
 (f,g:(mor x u); Hf:(sieve s f); Hg:(sieve s g))
 f==g -> (p x f Hf)==(p x g Hg).
Proof.
  NewDestruct 1.
  Cut Hf==Hg.
  NewDestruct 1; Reflexivity.
  Apply proof_irrelevance.
Qed.  

Definition matching_family_eq [p,q:Matching_Family] : Prop :=
 (x:A; f:(mor x u); Hf:(sieve s f)) (p x f Hf)==(q x f Hf).

Lemma matching_family_extensionality : (p,q:Matching_Family)
 (matching_family_eq p q) -> p==q.
Proof.
  Unfold matching_family_eq.
  NewDestruct p; NewDestruct q; Simpl; Intros.
  Cut matching_family_data0==matching_family_data1.
  NewDestruct 1.
  Cut ax_matching_family_prop0==ax_matching_family_prop1.
  NewDestruct 1; Reflexivity.
  Apply proof_irrelevance.
  Apply extensionality with T:=[x:A] (f:(mor x u))(sieve s f) -> (P x).
  Intros.
  Apply extensionality with T:=[f:(mor x u)](sieve s f) -> (P x).
  Auto.  
Qed.

Definition amalgamation_data [a:(P u)] : Matching_Family_Data :=
  [x:A; f:(mor x u); Hf:(sieve s f)](presh P f a).

Remark amalgamation_matches : (a:(P u))
 (Matching_Family_Prop (amalgamation_data a)).
Proof.
  Unfold Matching_Family_Prop amalgamation_data; Intros.
  Rewrite presh_oo; Reflexivity.
Qed.

Definition amalgamation [a:(P u)] : Matching_Family.
Proof.
  Intros; Split with (amalgamation_data a).
  Apply amalgamation_matches.
Defined.

End Matching_Family.

(** Let [p] be a matching family over a sieve [t].  For any
sub-sieve (sub-target) [s] of [t] we have a matching family
over [s] given by "retriction" of [p].  We call this
[restricted_matching_family]. *)

Section Matching_Family_Pullback.

Variable P:(Presheaf A Small_Set).
Variable u,v:A; f:(mor u v).
Variable s:(Sieve v); m:(Matching_Family P s).

Local s' : (Sieve u) := (sieve_transport f s).

Local h [a:A; r:(mor a u); Hr:(sieve s' r)] : (P a) :=
  (m ? ?  Hr).

Remark Hh : (Matching_Family_Prop h).
Proof.
  Unfold Matching_Family_Prop.
  Intros a r Hr y g Hrg; Intros.
  Unfold h.
  Assert Hrg' : (sieve s (oo (oo f r) g)).
  Rewrite <- oo_assoc.
  Apply Hrg.
  Transitivity (m y (oo (oo f r) g) Hrg').
  (* !!! this does not work.  
  Rewrite matching_family_prop.
  *)
  (* !!! if one wants to use Rewrite this is how to do...
  Assert HH := (matching_family_prop m Hr Hrg').
  Rewrite HH.
  Reflexivity.
  *)
  (* then we use apply, but it is not as good as rewrite... *)
  Apply matching_family_prop.
  Apply matching_family_congr.
  Auto.
Qed.

Definition matching_family_pullback : (Matching_Family P s').
Proof.
  Split with h.
  Exact Hh.
Defined.

End Matching_Family_Pullback.

Section Restricted_Matching_Family.

Variable u:A; P:(Presheaf A Small_Set).
Variable s,t:(Sieve u); p:(Matching_Family P t).
Hypothesis H:(target_sub s t).

Local m : (Matching_Family_Data P s).
Proof.
  Unfold Matching_Family_Data.
  Intros.
  NewDestruct p.
  EApply matching_family_data0.
  EApply H.
  Apply Hf.
Defined.

Remark Hm : (Matching_Family_Prop m).
Proof.
  Unfold Matching_Family_Prop; Intros.
  Assert Hf0 : (sieve t f).
  Apply H; Assumption.
  Unfold m.
  (* !!! Too difficult! *)
  Assert HH := (matching_family_prop p (H Hf)).
  NewDestruct p.
  Rewrite <- HH.
  Reflexivity.
Qed.

Definition restricted_matching_family : (Matching_Family P s).
Proof.
  Split with m; Exact Hm.
Defined.

End Restricted_Matching_Family.

(** A separated presheaf is a sheaf where every matching
family has at most one amalgamation.  Here we give a
constructive version of the definition using [cosection]
instead of injectivity. *)

Definition Separated_Presheaf [P:(Presheaf A Small_Set)] :=
 (u:A; s:(Sieve u); Hs:(cov_obj A s))
  (cosection [a:(P u)](amalgamation s a)). 

Record Sheaf : Type := {
  sheaf_presheaf :> (Presheaf A Small_Set);
  sheaf_patch : (u:A; s:(Sieve u); Hs:(cov_obj A s))
   (inverse [a:(sheaf_presheaf u)](amalgamation s a))
 }.

End Sheaf.

End Sheaf_.
