(* tag: Targets of a category *)

(** * Basis for Grothendieck topologies *)

Require cat.
Require fibprod.
Require misc.
Require poset.

Implicit Arguments On.

Module Target.

Export Cat.
Import Poset.
Import Misc.

Section Target.

Variable A:Cat; u:A.

Definition Target : Type := (x:A; f:(mor x u)) Prop.

Record Target_Set [a:Target] : Type := target_intro {
  target_src : A;
  target_mor :> (mor target_src u);
  target_prop : (a target_src target_mor)
 }.

Coercion Target_Set : Target >-> SORTCLASS.

Definition is_maximal_target [a:Target] : Prop :=
 (x:A; f:(mor x u)) (a ? f).

End Target.

Hints Unfold is_maximal_target.

Section Target_Transport.

Variable A:Cat; u,v:A; f:(mor u v); a:(Target v).

Definition target_transport : (Target u) :=
 [x:A; g:(mor x u)] (a (oo f g)).

End Target_Transport.

Section Target_Eq.

Variable A:Cat; u:A.

Definition target_sub [a,b:(Target u)] : Prop :=
 (x:A; f:(mor x u); Hf:(a ? f)) (b ? f).

Lemma target_extens : (a,b:(Target u))
 (target_sub a b) -> (target_sub b a) -> a==b.
Proof.
  Hints Resolve prop_uni.
  Unfold target_sub; Intros.
  Apply extensionality with T:=[x:A](f:(mor x u))Prop.
  Auto.
Qed.

End Target_Eq.

Hints Unfold target_sub.

Section Target_Poset.

Variable A:Cat; u:A.

Local X := (Target u).

Local leqX [s,t:X] : Prop := (target_sub s t).

Remark HX : (Ax_Poset leqX).
Proof.
  Unfold Ax_Poset leqX; Intuition.
  Unfold X.
  Unfold Ax_Anti_Symmetry.
  Intros.
  Apply target_extens; Trivial.
Qed.

Definition target_poset : Poset.
Proof.
  Split with X leqX; Exact HX.
Defined.

End Target_Poset.

Lemma target_transport_trans :
 (A:Cat; u,v,w:A; g:(mor v w); f:(mor u v); a:(Target w))
 (target_transport (oo g f) a)==(target_transport f (target_transport g a)).
Proof.
  Intros; Apply target_extens;
    Unfold target_sub target_transport; Intros; Simpl.
  Rewrite oo_assoc; Auto.
  Rewrite <- oo_assoc; Auto.
Qed.
Hints Resolve target_transport_trans.

Section Target_Composition.

Variable A:Cat; u:A; a:(Target u).
Variable b:(x:A; f:(mor x u); Hf:(a f))(Target x).

Inductive target_comp : (Target u) :=
 | target_comp_intro : (x:A; f:(mor x u); Hf:(a f))
    (y:A; g:(mor y x); Hg:((b Hf) y g))
    (target_comp (oo f g)).

End Target_Composition.

(* !!!!
Section Target_Composition.

Variable A:Cat; u:A; a:(Target u).
Variable b:(f:a)(Target (src f)).

Inductive target_comp : (Target u) :=
 | target_comp_intro : (x:A; f:(mor x u); Hf:(a f))
    (y:A; g:(mor y x); Hg:((!b (target_intro Hf)) ? g))
    (target_comp (oo f g)).

Definition make_target_comp [
  x1:A; f1:(mor x1 u); Hf1:(a f1); x2:A; f2:(mor x2 u); Hf2:(a f2);
  y:A; g:(mor y x2); Hg:((!b (target_intro Hf2)) ? g);
  Hf12:(eq_mor f1 f2); H:x2==x1
 ] : (target_comp (oo_weak f1 g H)).
Proof.
  Intros.
  Assert W := (!oo_weak_sx_eq A ? ? f1 ? ? g H f2 Hf12).
  Rewrite W.
  Split with Hf2.
  Exact Hg.
Defined.

End Target_Composition.
*)

(* !!!!
Section Target_Composition_Sub.

Variable A:Cat; u:A; a1,a2:(Target u); Ha:(target_sub a1 a2).
Variable b1,b2:(f:a2)(Target (src f)).
Variable Hb:(f:a2) (target_sub (!b1 f) (!b2 f)).

Local a2_of_a1 [f:a1] : a2.
Proof.
  NewDestruct 1.
  Split with target_mor := target_mor0.
  Apply Ha.
  Exact target_prop0.
Defined.

Lemma a2_of_a1_src : (f:a1) (target_src (a2_of_a1 f))==(target_src f).
Proof.
  NewDestruct f.
  Simpl.
  Reflexivity.
Qed.
 
Local b11 [f:a1] : (Target (src f)).
Proof.
  Unfold src; Intros.
  Assert TT  := (!b1 (a2_of_a1 f)).
  Unfold src in TT.
  Rewrite a2_of_a1_src in TT.
  Exact TT.
Defined.

Lemma target_comp_sub : (target_sub (target_comp b11) (target_comp b2)).
Proof.
  Intro; Intros.
  NewDestruct Hf.
  Assert Hf2 : (a2 f).
  Apply Ha.
  Exact Hf.
  Assert f2 := (!target_intro ? u a2 ? f Hf2).
  Split with Hf2.
  Apply Hb.
  Unfold b11 in Hg.

End Target_Composition_Sub.
*)

Section Target_Of_Mor.

Variable A:Cat; x,y:A; f:(mor x y).

Inductive target_of_mor  : (Target y) :=
 | target_of_mor_intro : (target_of_mor f).

End Target_Of_Mor.

Export Fibprod.

Section Pullback_Target.

Variable A:Cat; pb:(has_fibprod A).
Variable u:A; a:(Target u); v:A; h:(mor v u).

Inductive pullback_target : (Target v) :=
 | pullback_target_intro :
    (f:a)(pullback_target (fct (base u) (pullback (pb h f)))).

End Pullback_Target.

End Target.
