(* tag: Grothendieck topologies *)


Require Export cat_.
Require sieve_.
Require misc_.

Implicit Arguments On.

(** * Grothendieck Topologies. *)

Module Topology_.

Export Sieve_.
Import Misc_.

Syntactic Definition AX_Top_Maximal := [A:Cat; covs:(u:A)(Sieve u)->Type]
 (u:A; s:(Sieve u))
 (is_maximal_target s) -> (covs ? s) | 1.

Syntactic Definition AX_Top_Transport := [A:Cat; covs:(u:A)(Sieve u)->Type]
 (u,v:A; f:(mor u v); s:(Sieve v))
 (covs ? s) -> (covs ? (sieve_transport f s)) | 1.

Syntactic Definition AX_Top_Transitivity := [A:Cat; covs:(u:A)(Sieve u)->Type]
 (u:A; s,t:(Sieve u))
 (covs ? s) ->
 ((x:A; f:(mor x u); Hf:(sieve s f))(covs ? (sieve_transport f t))) ->
 (covs ? t) | 1.

Syntactic Definition AX_Top_Larger := [A:Cat; covs:(u:A)(Sieve u)->Type]
 (u:A; s,t:(Sieve u))
 (covs ? s) -> (target_sub s t) -> (covs ? t) | 1.

Syntactic Definition AX_Top_Comp := [A:Cat; covs:(u:A)(Sieve u)->Type]
 (u:A; s:(Sieve u); Hs:(covs ? s))
 (t:(x:A; f:(mor x u); Hf:(sieve s f))(Sieve x))
 (Ht:(x:A; f:(mor x u); Hf:(sieve s f))(covs ? (t x f Hf)))
 (covs ? (sieve_comp t)) | 1.

Section Topology.

Variable A:Cat.

Section Topology_Axioms.

Variable covs : (u:A)(Sieve u)->Type.

Definition Ax_Top_Maximal : Type := Eval Simpl in (AX_Top_Maximal covs).

Definition Ax_Top_Transport : Type := Eval Simpl in (AX_Top_Transport covs).

Definition Ax_Top_Transitivity : Type := Eval Simpl in (AX_Top_Transitivity covs).

Definition Ax_Top_Larger : Type := Eval Simpl in (AX_Top_Larger covs).

Definition Ax_Top_Comp : Type := Eval Simpl in (AX_Top_Comp covs).

Hints Unfold Ax_Top_Maximal Ax_Top_Transport Ax_Top_Transitivity
 Ax_Top_Larger Ax_Top_Comp.

Section Top_Larger_Prf.

Hypothesis top_maximal: Ax_Top_Maximal.
Hypothesis top_transport:Ax_Top_Transport.
Hypothesis top_transitivity:Ax_Top_Transitivity.

Local top_larger_goal := Eval Simpl in (AX_Top_Larger covs).
Lemma top_larger_prf : top_larger_goal.
Proof.
  Unfold top_larger_goal; Clear top_larger_goal; Intros.
  Unfold Ax_Top_Maximal Ax_Top_Transport Ax_Top_Transitivity in
    top_maximal top_transport top_transitivity.
  Apply top_transitivity with s.
  Assumption.
  Intros.
  Apply top_maximal.
  Apply is_maximal_sieve_ii.
  Simpl.
  Unfold target_transport.
  Apply H.
  Rewrite ii_dx.
  Auto.
Qed.

End Top_Larger_Prf.

Section Top_Comp_Prf.

Hypothesis top_maximal:Ax_Top_Maximal.
Hypothesis top_transport:Ax_Top_Transport.
Hypothesis top_transitivity:Ax_Top_Transitivity.

Local top_comp_goal := Eval Simpl in (AX_Top_Comp covs).
Lemma top_comp_prf : top_comp_goal.
Proof.
  Unfold top_comp_goal; Clear top_comp_goal; Intros.
  Unfold Ax_Top_Maximal Ax_Top_Transport Ax_Top_Transitivity in
    top_maximal top_transport top_transitivity.
  Apply top_transitivity with s.
  Assumption.
  Intros.
  Apply top_larger_prf with (t x f Hf); Try Assumption.
  Apply (Ht x f Hf).
  Unfold target_sub.
  Intros y g Hy.
  Split with Hf.
  Assumption.
Qed.

End Top_Comp_Prf.

Section Top_Transitivity_Prf.

Hypothesis top_maximal:Ax_Top_Maximal.
Hypothesis top_larger:Ax_Top_Larger.
Hypothesis top_transport:Ax_Top_Transport.
Hypothesis top_comp:Ax_Top_Comp.

Local top_transitivity_goal := Eval Simpl in (AX_Top_Transitivity covs).
Lemma top_transitivity_prf : top_transitivity_goal.
Proof.
  Unfold top_transitivity_goal; Clear top_transitivity_goal.
  Unfold Ax_Top_Maximal Ax_Top_Larger Ax_Top_Transport Ax_Top_Comp in
    top_maximal top_larger top_transport top_comp.
  Intros u s t Hs Hf. 
  Apply top_larger with
   (sieve_comp [x:A; f:(mor x u); Hf:(sieve s f)](sieve_transport f t)).
  Apply top_comp.
  Assumption.
  Auto.
  Unfold target_sub.
  NewDestruct 1; Auto.
Qed.

End Top_Transitivity_Prf.

End Topology_Axioms.

Hints Unfold Ax_Top_Maximal Ax_Top_Transport Ax_Top_Transitivity
 Ax_Top_Larger Ax_Top_Comp.

Record Topology : Type := {
  cov_obj : (u:A)(Sieve u)->Type;
  (* !!! ma qui gli ax_ a cosa servono? *)
  ax_top_Maximal : (Ax_Top_Maximal cov_obj);
  ax_top_transport : (Ax_Top_Transport cov_obj);
  ax_top_transitivity : (Ax_Top_Transitivity cov_obj)
 }.

Section Topology_Facts.

Variable J:Topology.

(** The maximal sieve is a covering sieve. *)
Lemma top_maximal : (u:A; s:(Sieve u))
 (is_maximal_target s) -> (cov_obj J s).
Proof.
  Elim J; Simpl; Auto.
Qed.

(** Transport property. *)
Lemma top_transport : (u,v:A; f:(mor u v); s:(Sieve v))
 (cov_obj J s) -> (cov_obj J (sieve_transport f s)).
Proof.
  Elim J; Simpl; Auto.
Qed.

(** Transitivity property. *)
Lemma top_transitivity : (u:A; s,t:(Sieve u))
 (cov_obj J s) ->
 ((x:A; f:(mor x u); Hf:(sieve s f))(cov_obj J (sieve_transport f t)))
  -> (cov_obj J t).
Proof.
  Elim J; Simpl; Auto.
Qed.

Hints Resolve top_maximal top_transport top_transitivity.
Hints Unfold Ax_Top_Maximal Ax_Top_Transport Ax_Top_Transitivity.

(** Any larger sieve of a covering sieve is also a covering sieve. *)
Lemma top_larger : (u:A; s,t:(Sieve u))
 (cov_obj J s) -> (target_sub s t) -> (cov_obj J t).
Proof.
  Assert top_larger := top_larger_prf.
  Unfold Ax_Top_Larger in top_larger.
  Intros.
  Apply top_larger with s; EAuto.
Qed.

Hints Resolve top_larger.

(** Composition property. *)
Lemma top_comp :
 (u:A; s:(Sieve u); Hs:(cov_obj J s))
 (t:(x:A; f:(mor x u); Hf:(sieve s f))(Sieve (src f)))
 (Ht:(x:A; f:(mor x u); Hf:(sieve s f))(cov_obj J (t x f Hf)))
 (cov_obj J (sieve_comp t)).
Proof.
  Assert top_comp := top_comp_prf.
  Unfold Ax_Top_Comp in top_comp.
  Intros.
  Apply top_comp; EAuto.
Qed.

Hints Resolve top_comp.

Lemma cov_obj_transport: (u:A; s:(Sieve u))
 (x:A; f:(mor x u); Hf:(sieve s f))
 (cov_obj J (sieve_transport f s)).
Proof.
  Intros.
  Apply top_maximal.
  Apply is_maximal_sieve_ii; Simpl.
  Unfold target_transport.
  Rewrite ii_dx.
  Auto.
Qed.

End Topology_Facts.

Hints Resolve top_maximal top_larger top_transport top_comp.
Hints Resolve top_transitivity cov_obj_transport.

Section Top_Intro_Bis.

Variable covs : (u:A)(Sieve u)->Type.
Hypothesis top_maximal : (Ax_Top_Maximal covs).
Hypothesis top_larger : (Ax_Top_Larger covs).
Hypothesis top_transport : (Ax_Top_Transport covs).
Hypothesis top_comp : (Ax_Top_Comp covs).

Definition top_intro_bis : Topology.
Proof.
  Split with covs; Try Assumption.
  Unfold Ax_Top_Transitivity.
  Apply top_transitivity_prf; Try Assumption.
Defined.

Remark top_intro_bis_ck : covs==(cov_obj top_intro_bis).
Proof.
  Reflexivity.
Qed.

End Top_Intro_Bis.

Section Cov_Sieve.

Variable J:Topology; u:A; s:(Sieve u).

Local cov_sieve_data [x:A; f:(mor x u)] : Prop :=
 (prop_of_type (cov_obj J (sieve_transport f s))).

Remark cov_sieve_theory : (Sieve_Theory cov_sieve_data).
Proof.
  Unfold cov_sieve_data Sieve_Theory; Intros.
  Rewrite sieve_transport_trans.
  NewDestruct H.
  Auto.
Qed.

Definition Cov : (Sieve u).
Proof.
  Split with cov_sieve_data.
  Exact cov_sieve_theory.
Defined.

Lemma cov_sieve_geq : (target_sub s Cov).
Proof.
  Unfold target_sub; Simpl.
  Unfold cov_sieve_data; Auto.
Qed.

Hints Resolve cov_sieve_geq.

Definition cov [x:A; f:(mor x u)] : Prop := (sieve Cov f).

End Cov_Sieve.

Lemma top_trans_arrow : (J:Topology; u:A; s,t:(Sieve u); f:s)
 ((g:s)(cov J t g)) -> (cov J t f).
Proof.
  Unfold cov; Auto.
Qed.

Hints Resolve top_trans_arrow.

Lemma top_coerence : (J:Topology; u:A; s:(Sieve u); x:A; f:s)
 (cov J s f).
Proof.
  Intros; NewDestruct f; Unfold cov; Simpl; Auto.
Qed.

Hints Resolve top_coerence.

End Topology.

Hints Resolve top_maximal top_transport top_comp top_transitivity.
Hints Resolve sieve_prop.

(** ** Sites *)

(** A site is a category with a Grothendieck topology. *)

Record Site : Type := {
  site_cat :> Cat;
  site_top :> (Topology site_cat)
 }.

End Topology_.
